@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -73,7 +73,15 @@ enum AbortReason {
     RepatchInsaneArgumentCount                        = 310,
     TGInvalidPointer                                  = 320,
     TGNotSupported                                    = 330,
-    YARRNoInputConsumed                               = 340,
+    UncheckedOverflow                                 = 335,
+};
+
+// This enum is for CRASH_WITH_SECURITY_IMPLICATION_AND_INFO so we can easily identify which assertion
+// we are looking at even if the calls to crash get coalesed. The same numbering rules above for
+// AbortReason apply here.
+enum CompilerAbortReason {
+    AbstractInterpreterInvalidType                =  10,
+    ObjectAllocationSinkingAssertionFailure       = 100,
 };
 
 } // namespace JSC
