@@ -28,13 +28,10 @@
 #if ENABLE(DFG_JIT)
 
 #include "DFGAbstractValue.h"
-#include "DFGAvailability.h"
 #include "DFGAvailabilityMap.h"
 #include "DFGBranchDirection.h"
-#include "DFGFlushedAt.h"
 #include "DFGNode.h"
 #include "DFGNodeAbstractValuePair.h"
-#include "DFGNodeOrigin.h"
 #include "DFGStructureClobberState.h"
 #include "Operands.h"
 #include <wtf/Vector.h>
@@ -67,6 +64,11 @@ struct BasicBlock : RefCounted<BasicBlock> {
     }
     Node*& operator[](size_t i) { return at(i); }
     Node* operator[](size_t i) const { return at(i); }
+    Node* last() const
+    {
+        RELEASE_ASSERT(!!size());
+        return at(size() - 1);
+    }
     
     // Use this to find both the index of the terminal and the terminal itself in one go. May
     // return a clear NodeAndIndex if the basic block currently lacks a terminal. That may happen
@@ -85,19 +87,12 @@ struct BasicBlock : RefCounted<BasicBlock> {
         size_t i = size();
         while (i--) {
             Node* node = at(i);
-            switch (node->op()) {
-            case Jump:
-            case Branch:
-            case Switch:
-            case Return:
-            case TailCall:
-            case DirectTailCall:
-            case TailCallVarargs:
-            case TailCallForwardVarargs:
-            case Unreachable:
+            if (node->isTerminal())
                 return NodeAndIndex(node, i);
+            switch (node->op()) {
             // The bitter end can contain Phantoms and the like. There will probably only be one or two nodes after the terminal. They are all no-ops and will not have any checked children.
             case Check: // This is here because it's our universal no-op.
+            case CheckVarargs:
             case Phantom:
             case PhantomLocal:
             case Flush:
@@ -127,7 +122,7 @@ struct BasicBlock : RefCounted<BasicBlock> {
             m_nodes.insert(result.index, node);
     }
     
-    void replaceTerminal(Node*);
+    void replaceTerminal(Graph&, Node*);
     
     size_t numNodes() const { return phis.size() + size(); }
     Node* node(size_t i) const
@@ -186,15 +181,18 @@ struct BasicBlock : RefCounted<BasicBlock> {
     unsigned bytecodeBegin;
     
     BlockIndex index;
-    
-    bool isOSRTarget;
+
+    StructureClobberState cfaStructureClobberStateAtHead;
+    StructureClobberState cfaStructureClobberStateAtTail;
+    BranchDirection cfaBranchDirection;
     bool cfaHasVisited;
     bool cfaShouldRevisit;
     bool cfaFoundConstants;
     bool cfaDidFinish;
-    StructureClobberState cfaStructureClobberStateAtHead;
-    StructureClobberState cfaStructureClobberStateAtTail;
-    BranchDirection cfaBranchDirection;
+    bool intersectionOfCFAHasVisited;
+    bool isOSRTarget;
+    bool isCatchEntrypoint;
+
 #if !ASSERT_DISABLED
     bool isLinked;
 #endif
@@ -227,14 +225,9 @@ struct BasicBlock : RefCounted<BasicBlock> {
     // would not be a productive optimization: it would make setting up a basic block more
     // expensive and would only benefit bizarre pathological cases.
     Operands<AbstractValue> intersectionOfPastValuesAtHead;
-    bool intersectionOfCFAHasVisited;
     
     float executionCount;
     
-    // These fields are reserved for NaturalLoops.
-    static const unsigned numberOfInnerMostLoopIndices = 2;
-    unsigned innerMostLoopIndices[numberOfInnerMostLoopIndices];
-
     struct SSAData {
         WTF_MAKE_FAST_ALLOCATED;
     public:
@@ -264,22 +257,7 @@ private:
     BlockNodeList m_nodes;
 };
 
-typedef Vector<BasicBlock*, 5> BlockList;
-
-struct UnlinkedBlock {
-    BasicBlock* m_block;
-    bool m_needsNormalLinking;
-    bool m_needsEarlyReturnLinking;
-    
-    UnlinkedBlock() { }
-    
-    explicit UnlinkedBlock(BasicBlock* block)
-        : m_block(block)
-        , m_needsNormalLinking(true)
-        , m_needsEarlyReturnLinking(false)
-    {
-    }
-};
+typedef Vector<BasicBlock*> BlockList;
     
 static inline unsigned getBytecodeBeginForBlock(BasicBlock** basicBlock)
 {
