@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2015-2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,33 +26,43 @@
 #include "config.h"
 #include "StackVisitor.h"
 
+#include "CallFrameInlines.h"
 #include "ClonedArguments.h"
 #include "DebuggerPrimitives.h"
-#include "Executable.h"
 #include "InlineCallFrame.h"
 #include "Interpreter.h"
 #include "JSCInlines.h"
+#include "WasmCallee.h"
+#include "WasmIndexOrName.h"
+#include "WebAssemblyFunction.h"
 #include <wtf/text/StringBuilder.h>
 
 namespace JSC {
 
-StackVisitor::StackVisitor(CallFrame* startFrame)
+StackVisitor::StackVisitor(CallFrame* startFrame, VM* vm)
 {
     m_frame.m_index = 0;
+    m_frame.m_isWasmFrame = false;
     CallFrame* topFrame;
     if (startFrame) {
-        m_frame.m_VMEntryFrame = startFrame->vm().topVMEntryFrame;
-        topFrame = startFrame->vm().topCallFrame;
-        
-        if (topFrame && static_cast<void*>(m_frame.m_VMEntryFrame) == static_cast<void*>(topFrame)) {
-            topFrame = vmEntryRecord(m_frame.m_VMEntryFrame)->m_prevTopCallFrame;
-            m_frame.m_VMEntryFrame = vmEntryRecord(m_frame.m_VMEntryFrame)->m_prevTopVMEntryFrame;
+        ASSERT(vm);
+        ASSERT(!vm->topCallFrame || reinterpret_cast<void*>(vm->topCallFrame) != vm->topEntryFrame);
+
+        m_frame.m_entryFrame = vm->topEntryFrame;
+        topFrame = vm->topCallFrame;
+
+        if (topFrame && topFrame->isStackOverflowFrame()) {
+            topFrame = topFrame->callerFrame(m_frame.m_entryFrame);
+            m_topEntryFrameIsEmpty = (m_frame.m_entryFrame != vm->topEntryFrame);
+            if (startFrame == vm->topCallFrame)
+                startFrame = topFrame;
         }
+
     } else {
-        m_frame.m_VMEntryFrame = 0;
+        m_frame.m_entryFrame = 0;
         topFrame = 0;
     }
-    m_frame.m_callerIsVMEntryFrame = false;
+    m_frame.m_callerIsEntryFrame = false;
     readFrame(topFrame);
 
     // Find the frame the caller wants to start unwinding from.
@@ -72,14 +82,14 @@ void StackVisitor::gotoNextFrame()
                 readInlinedFrame(m_frame.callFrame(), &inlineCallFrame->directCaller);
                 inlineCallFrame = m_frame.inlineCallFrame();
             }
-            m_frame.m_VMEntryFrame = m_frame.m_CallerVMEntryFrame;
+            m_frame.m_entryFrame = m_frame.m_callerEntryFrame;
             readFrame(m_frame.callerFrame());
         } else
             readInlinedFrame(m_frame.callFrame(), callerCodeOrigin);
         return;
     }
 #endif // ENABLE(DFG_JIT)
-    m_frame.m_VMEntryFrame = m_frame.m_CallerVMEntryFrame;
+    m_frame.m_entryFrame = m_frame.m_callerEntryFrame;
     readFrame(m_frame.callerFrame());
 }
 
@@ -88,8 +98,8 @@ void StackVisitor::unwindToMachineCodeBlockFrame()
 #if ENABLE(DFG_JIT)
     if (m_frame.isInlinedFrame()) {
         CodeOrigin codeOrigin = m_frame.inlineCallFrame()->directCaller;
-        while (codeOrigin.inlineCallFrame)
-            codeOrigin = codeOrigin.inlineCallFrame->directCaller;
+        while (codeOrigin.inlineCallFrame())
+            codeOrigin = codeOrigin.inlineCallFrame()->directCaller;
         readNonInlinedFrame(m_frame.callFrame(), &codeOrigin);
     }
 #endif
@@ -102,6 +112,11 @@ void StackVisitor::readFrame(CallFrame* callFrame)
         return;
     }
 
+    if (callFrame->isAnyWasmCallee()) {
+        readNonInlinedFrame(callFrame);
+        return;
+    }
+
 #if !ENABLE(DFG_JIT)
     readNonInlinedFrame(callFrame);
 
@@ -131,7 +146,7 @@ void StackVisitor::readFrame(CallFrame* callFrame)
     }
 
     CodeOrigin codeOrigin = codeBlock->codeOrigin(index);
-    if (!codeOrigin.inlineCallFrame) {
+    if (!codeOrigin.inlineCallFrame()) {
         readNonInlinedFrame(callFrame, &codeOrigin);
         return;
     }
@@ -144,14 +159,31 @@ void StackVisitor::readNonInlinedFrame(CallFrame* callFrame, CodeOrigin* codeOri
 {
     m_frame.m_callFrame = callFrame;
     m_frame.m_argumentCountIncludingThis = callFrame->argumentCountIncludingThis();
-    m_frame.m_CallerVMEntryFrame = m_frame.m_VMEntryFrame;
-    m_frame.m_callerFrame = callFrame->callerFrame(m_frame.m_CallerVMEntryFrame);
-    m_frame.m_callerIsVMEntryFrame = m_frame.m_CallerVMEntryFrame != m_frame.m_VMEntryFrame;
-    m_frame.m_callee = callFrame->callee();
-    m_frame.m_codeBlock = callFrame->codeBlock();
-    m_frame.m_bytecodeOffset = !m_frame.codeBlock() ? 0
-        : codeOrigin ? codeOrigin->bytecodeIndex
-        : callFrame->bytecodeOffset();
+    m_frame.m_callerEntryFrame = m_frame.m_entryFrame;
+    m_frame.m_callerFrame = callFrame->callerFrame(m_frame.m_callerEntryFrame);
+    m_frame.m_callerIsEntryFrame = m_frame.m_callerEntryFrame != m_frame.m_entryFrame;
+    m_frame.m_isWasmFrame = false;
+
+    CalleeBits callee = callFrame->callee();
+    m_frame.m_callee = callee;
+
+    if (callFrame->isAnyWasmCallee()) {
+        m_frame.m_isWasmFrame = true;
+        m_frame.m_codeBlock = nullptr;
+        m_frame.m_bytecodeOffset = 0;
+#if ENABLE(WEBASSEMBLY)
+        CalleeBits bits = callFrame->callee();
+        if (bits.isWasm())
+            m_frame.m_wasmFunctionIndexOrName = bits.asWasmCallee()->indexOrName();
+#endif
+    } else {
+        m_frame.m_codeBlock = callFrame->codeBlock();
+        m_frame.m_bytecodeOffset = !m_frame.codeBlock() ? 0
+            : codeOrigin ? codeOrigin->bytecodeIndex()
+            : callFrame->bytecodeOffset();
+
+    }
+
 #if ENABLE(DFG_JIT)
     m_frame.m_inlineCallFrame = 0;
 #endif
@@ -160,7 +192,7 @@ void StackVisitor::readNonInlinedFrame(CallFrame* callFrame, CodeOrigin* codeOri
 #if ENABLE(DFG_JIT)
 static int inlinedFrameOffset(CodeOrigin* codeOrigin)
 {
-    InlineCallFrame* inlineCallFrame = codeOrigin->inlineCallFrame;
+    InlineCallFrame* inlineCallFrame = codeOrigin->inlineCallFrame();
     int frameOffset = inlineCallFrame ? inlineCallFrame->stackOffset : 0;
     return frameOffset;
 }
@@ -168,24 +200,25 @@ static int inlinedFrameOffset(CodeOrigin* codeOrigin)
 void StackVisitor::readInlinedFrame(CallFrame* callFrame, CodeOrigin* codeOrigin)
 {
     ASSERT(codeOrigin);
+    m_frame.m_isWasmFrame = false;
 
     int frameOffset = inlinedFrameOffset(codeOrigin);
     bool isInlined = !!frameOffset;
     if (isInlined) {
-        InlineCallFrame* inlineCallFrame = codeOrigin->inlineCallFrame;
+        InlineCallFrame* inlineCallFrame = codeOrigin->inlineCallFrame();
 
         m_frame.m_callFrame = callFrame;
         m_frame.m_inlineCallFrame = inlineCallFrame;
         if (inlineCallFrame->argumentCountRegister.isValid())
             m_frame.m_argumentCountIncludingThis = callFrame->r(inlineCallFrame->argumentCountRegister.offset()).unboxedInt32();
         else
-            m_frame.m_argumentCountIncludingThis = inlineCallFrame->arguments.size();
+            m_frame.m_argumentCountIncludingThis = inlineCallFrame->argumentCountIncludingThis;
         m_frame.m_codeBlock = inlineCallFrame->baselineCodeBlock.get();
-        m_frame.m_bytecodeOffset = codeOrigin->bytecodeIndex;
+        m_frame.m_bytecodeOffset = codeOrigin->bytecodeIndex();
 
         JSFunction* callee = inlineCallFrame->calleeForCallFrame(callFrame);
         m_frame.m_callee = callee;
-        ASSERT(m_frame.callee());
+        ASSERT(!!m_frame.callee().rawPtr());
 
         // The callerFrame just needs to be non-null to indicate that we
         // haven't reached the last frame yet. Setting it to the root
@@ -201,7 +234,10 @@ void StackVisitor::readInlinedFrame(CallFrame* callFrame, CodeOrigin* codeOrigin
 
 StackVisitor::Frame::CodeType StackVisitor::Frame::codeType() const
 {
-    if (!isJSFrame())
+    if (isWasmFrame())
+        return CodeType::Wasm;
+
+    if (!codeBlock())
         return CodeType::Native;
 
     switch (codeBlock()->codeType()) {
@@ -218,27 +254,63 @@ StackVisitor::Frame::CodeType StackVisitor::Frame::codeType() const
     return CodeType::Global;
 }
 
+#if ENABLE(ASSEMBLER)
+Optional<RegisterAtOffsetList> StackVisitor::Frame::calleeSaveRegistersForUnwinding()
+{
+    if (!NUMBER_OF_CALLEE_SAVES_REGISTERS)
+        return WTF::nullopt;
+
+    if (isInlinedFrame())
+        return WTF::nullopt;
+
+#if ENABLE(WEBASSEMBLY)
+    if (isWasmFrame()) {
+        if (callee().isCell()) {
+            RELEASE_ASSERT(isWebAssemblyToJSCallee(callee().asCell()));
+            return WTF::nullopt;
+        }
+        Wasm::Callee* wasmCallee = callee().asWasmCallee();
+        return *wasmCallee->calleeSaveRegisters();
+    }
+
+    if (callee().isCell()) {
+        if (auto* jsToWasmICCallee = jsDynamicCast<JSToWasmICCallee*>(callee().asCell()->vm(), callee().asCell()))
+            return jsToWasmICCallee->function()->usedCalleeSaveRegisters();
+    }
+#endif // ENABLE(WEBASSEMBLY)
+
+    if (CodeBlock* codeBlock = this->codeBlock())
+        return *codeBlock->calleeSaveRegisters();
+
+    return WTF::nullopt;
+}
+#endif // ENABLE(ASSEMBLER)
+
 String StackVisitor::Frame::functionName() const
 {
     String traceLine;
-    JSObject* callee = this->callee();
 
     switch (codeType()) {
+    case CodeType::Wasm:
+        traceLine = makeString(m_wasmFunctionIndexOrName);
+        break;
     case CodeType::Eval:
-        traceLine = ASCIILiteral("eval code");
+        traceLine = "eval code"_s;
         break;
     case CodeType::Module:
-        traceLine = ASCIILiteral("module code");
+        traceLine = "module code"_s;
         break;
-    case CodeType::Native:
+    case CodeType::Native: {
+        JSCell* callee = this->callee().asCell();
         if (callee)
-            traceLine = getCalculatedDisplayName(callFrame()->vm(), callee).impl();
+            traceLine = getCalculatedDisplayName(callFrame()->vm(), jsCast<JSObject*>(callee)).impl();
         break;
-    case CodeType::Function:
-        traceLine = getCalculatedDisplayName(callFrame()->vm(), callee).impl();
+    }
+    case CodeType::Function: 
+        traceLine = getCalculatedDisplayName(callFrame()->vm(), jsCast<JSObject*>(this->callee().asCell())).impl();
         break;
     case CodeType::Global:
-        traceLine = ASCIILiteral("global code");
+        traceLine = "global code"_s;
         break;
     }
     return traceLine.isNull() ? emptyString() : traceLine;
@@ -253,13 +325,16 @@ String StackVisitor::Frame::sourceURL() const
     case CodeType::Module:
     case CodeType::Function:
     case CodeType::Global: {
-        String sourceURL = codeBlock()->ownerScriptExecutable()->sourceURL();
+        String sourceURL = codeBlock()->ownerExecutable()->sourceURL();
         if (!sourceURL.isEmpty())
             traceLine = sourceURL.impl();
         break;
     }
     case CodeType::Native:
-        traceLine = ASCIILiteral("[native code]");
+        traceLine = "[native code]"_s;
+        break;
+    case CodeType::Wasm:
+        traceLine = "[wasm code]"_s;
         break;
     }
     return traceLine.isNull() ? emptyString() : traceLine;
@@ -275,7 +350,7 @@ String StackVisitor::Frame::toString() const
         if (!functionName.isEmpty())
             traceBuild.append('@');
         traceBuild.append(sourceURL);
-        if (isJSFrame()) {
+        if (hasLineAndColumnInfo()) {
             unsigned line = 0;
             unsigned column = 0;
             computeLineAndColumn(line, column);
@@ -291,7 +366,7 @@ String StackVisitor::Frame::toString() const
 intptr_t StackVisitor::Frame::sourceID()
 {
     if (CodeBlock* codeBlock = this->codeBlock())
-        return codeBlock->ownerScriptExecutable()->sourceID();
+        return codeBlock->ownerExecutable()->sourceID();
     return noSourceID;
 }
 
@@ -315,6 +390,11 @@ ClonedArguments* StackVisitor::Frame::createArguments()
     return arguments;
 }
 
+bool StackVisitor::Frame::hasLineAndColumnInfo() const
+{
+    return !!codeBlock();
+}
+
 void StackVisitor::Frame::computeLineAndColumn(unsigned& line, unsigned& column) const
 {
     CodeBlock* codeBlock = this->codeBlock();
@@ -331,11 +411,11 @@ void StackVisitor::Frame::computeLineAndColumn(unsigned& line, unsigned& column)
     unsigned divotColumn = 0;
     retrieveExpressionInfo(divot, unusedStartOffset, unusedEndOffset, divotLine, divotColumn);
 
-    line = divotLine + codeBlock->ownerScriptExecutable()->firstLine();
+    line = divotLine + codeBlock->ownerExecutable()->firstLine();
     column = divotColumn + (divotLine ? 1 : codeBlock->firstLineColumnOffset());
 
-    if (codeBlock->ownerScriptExecutable()->hasOverrideLineNumber())
-        line = codeBlock->ownerScriptExecutable()->overrideLineNumber();
+    if (Optional<int> overrideLineNumber = codeBlock->ownerExecutable()->overrideLineNumber(codeBlock->vm()))
+        line = overrideLineNumber.value();
 }
 
 void StackVisitor::Frame::retrieveExpressionInfo(int& divot, int& startOffset, int& endOffset, unsigned& line, unsigned& column) const
@@ -351,6 +431,7 @@ void StackVisitor::Frame::setToEnd()
 #if ENABLE(DFG_JIT)
     m_inlineCallFrame = 0;
 #endif
+    m_isWasmFrame = false;
 }
 
 void StackVisitor::Frame::dump(PrintStream& out, Indenter indent) const
@@ -358,7 +439,7 @@ void StackVisitor::Frame::dump(PrintStream& out, Indenter indent) const
     dump(out, indent, [] (PrintStream&) { });
 }
 
-void StackVisitor::Frame::dump(PrintStream& out, Indenter indent, std::function<void(PrintStream&)> prefix) const
+void StackVisitor::Frame::dump(PrintStream& out, Indenter indent, WTF::Function<void(PrintStream&)> prefix) const
 {
     if (!this->callFrame()) {
         out.print(indent, "frame 0x0\n");
@@ -375,7 +456,7 @@ void StackVisitor::Frame::dump(PrintStream& out, Indenter indent, std::function<
 
         CallFrame* callFrame = m_callFrame;
         CallFrame* callerFrame = this->callerFrame();
-        void* returnPC = callFrame->hasReturnPC() ? callFrame->returnPC().value() : nullptr;
+        const void* returnPC = callFrame->hasReturnPC() ? callFrame->returnPC().value() : nullptr;
 
         out.print(indent, "name: ", functionName(), "\n");
         out.print(indent, "sourceURL: ", sourceURL(), "\n");
@@ -388,15 +469,15 @@ void StackVisitor::Frame::dump(PrintStream& out, Indenter indent, std::function<
             out.print(indent, "InlineCallFrame: ", RawPointer(m_inlineCallFrame), "\n");
 #endif
 
-        out.print(indent, "callee: ", RawPointer(callee()), "\n");
+        out.print(indent, "callee: ", RawPointer(callee().rawPtr()), "\n");
         out.print(indent, "returnPC: ", RawPointer(returnPC), "\n");
         out.print(indent, "callerFrame: ", RawPointer(callerFrame), "\n");
-        unsigned locationRawBits = callFrame->callSiteAsRawBits();
-        out.print(indent, "rawLocationBits: ", static_cast<uintptr_t>(locationRawBits),
+        uintptr_t locationRawBits = callFrame->callSiteAsRawBits();
+        out.print(indent, "rawLocationBits: ", locationRawBits,
             " ", RawPointer(reinterpret_cast<void*>(locationRawBits)), "\n");
         out.print(indent, "codeBlock: ", RawPointer(codeBlock));
         if (codeBlock)
-            out.print(*codeBlock);
+            out.print(" ", *codeBlock);
         out.print("\n");
         if (codeBlock && !isInlined) {
             indent++;
@@ -411,8 +492,8 @@ void StackVisitor::Frame::dump(PrintStream& out, Indenter indent, std::function<
                     CallSiteIndex callSiteIndex = callFrame->callSiteIndex();
                     out.print(indent, "callSiteIndex: ", callSiteIndex.bits(), " of ", codeBlock->codeOrigins().size(), "\n");
 
-                    JITCode::JITType jitType = codeBlock->jitType();
-                    if (jitType != JITCode::FTLJIT) {
+                    JITType jitType = codeBlock->jitType();
+                    if (jitType != JITType::FTLJIT) {
                         JITCode* jitCode = codeBlock->jitCode().get();
                         out.print(indent, "jitCode: ", RawPointer(jitCode),
                             " start ", RawPointer(jitCode->start()),
@@ -429,7 +510,7 @@ void StackVisitor::Frame::dump(PrintStream& out, Indenter indent, std::function<
 
             indent--;
         }
-        out.print(indent, "vmEntryFrame: ", RawPointer(vmEntryFrame()), "\n");
+        out.print(indent, "EntryFrame: ", RawPointer(m_entryFrame), "\n");
         indent--;
     }
     out.print(indent, "}\n");
