@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2009-2019 Apple Inc. All rights reserved.
  * Copyright (C) 2012 Mathias Bynens (mathias@qiwi.be)
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,6 +37,7 @@
 #include "StrongInlines.h"
 #include <wtf/ASCIICType.h>
 #include <wtf/dtoa.h>
+#include <wtf/text/StringConcatenate.h>
 
 namespace JSC {
 
@@ -58,17 +59,17 @@ bool LiteralParser<CharType>::tryJSONPParse(Vector<JSONPData>& results, bool nee
     do {
         Vector<JSONPPathEntry> path;
         // Unguarded next to start off the lexer
-        Identifier name = Identifier::fromString(&vm, m_lexer.currentToken()->start, m_lexer.currentToken()->end - m_lexer.currentToken()->start);
+        Identifier name = Identifier::fromString(vm, m_lexer.currentToken()->start, m_lexer.currentToken()->end - m_lexer.currentToken()->start);
         JSONPPathEntry entry;
         if (name == vm.propertyNames->varKeyword) {
             if (m_lexer.next() != TokIdentifier)
                 return false;
             entry.m_type = JSONPPathEntryTypeDeclareVar;
-            entry.m_pathEntryName = Identifier::fromString(&vm, m_lexer.currentToken()->start, m_lexer.currentToken()->end - m_lexer.currentToken()->start);
+            entry.m_pathEntryName = Identifier::fromString(vm, m_lexer.currentToken()->start, m_lexer.currentToken()->end - m_lexer.currentToken()->start);
             path.append(entry);
         } else {
             entry.m_type = JSONPPathEntryTypeDot;
-            entry.m_pathEntryName = Identifier::fromString(&vm, m_lexer.currentToken()->start, m_lexer.currentToken()->end - m_lexer.currentToken()->start);
+            entry.m_pathEntryName = Identifier::fromString(vm, m_lexer.currentToken()->start, m_lexer.currentToken()->end - m_lexer.currentToken()->start);
             path.append(entry);
         }
         if (isLexerKeyword(entry.m_pathEntryName))
@@ -95,7 +96,7 @@ bool LiteralParser<CharType>::tryJSONPParse(Vector<JSONPData>& results, bool nee
                 entry.m_type = JSONPPathEntryTypeDot;
                 if (m_lexer.next() != TokIdentifier)
                     return false;
-                entry.m_pathEntryName = Identifier::fromString(&vm, m_lexer.currentToken()->start, m_lexer.currentToken()->end - m_lexer.currentToken()->start);
+                entry.m_pathEntryName = Identifier::fromString(vm, m_lexer.currentToken()->start, m_lexer.currentToken()->end - m_lexer.currentToken()->start);
                 break;
             }
             case TokLParen: {
@@ -135,40 +136,42 @@ bool LiteralParser<CharType>::tryJSONPParse(Vector<JSONPData>& results, bool nee
 template <typename CharType>
 ALWAYS_INLINE const Identifier LiteralParser<CharType>::makeIdentifier(const LChar* characters, size_t length)
 {
+    VM& vm = m_exec->vm();
     if (!length)
-        return m_exec->vm().propertyNames->emptyIdentifier;
+        return vm.propertyNames->emptyIdentifier;
     if (characters[0] >= MaximumCachableCharacter)
-        return Identifier::fromString(&m_exec->vm(), characters, length);
+        return Identifier::fromString(vm, characters, length);
 
     if (length == 1) {
         if (!m_shortIdentifiers[characters[0]].isNull())
             return m_shortIdentifiers[characters[0]];
-        m_shortIdentifiers[characters[0]] = Identifier::fromString(&m_exec->vm(), characters, length);
+        m_shortIdentifiers[characters[0]] = Identifier::fromString(vm, characters, length);
         return m_shortIdentifiers[characters[0]];
     }
     if (!m_recentIdentifiers[characters[0]].isNull() && Identifier::equal(m_recentIdentifiers[characters[0]].impl(), characters, length))
         return m_recentIdentifiers[characters[0]];
-    m_recentIdentifiers[characters[0]] = Identifier::fromString(&m_exec->vm(), characters, length);
+    m_recentIdentifiers[characters[0]] = Identifier::fromString(vm, characters, length);
     return m_recentIdentifiers[characters[0]];
 }
 
 template <typename CharType>
 ALWAYS_INLINE const Identifier LiteralParser<CharType>::makeIdentifier(const UChar* characters, size_t length)
 {
+    VM& vm = m_exec->vm();
     if (!length)
-        return m_exec->vm().propertyNames->emptyIdentifier;
+        return vm.propertyNames->emptyIdentifier;
     if (characters[0] >= MaximumCachableCharacter)
-        return Identifier::fromString(&m_exec->vm(), characters, length);
+        return Identifier::fromString(vm, characters, length);
 
     if (length == 1) {
         if (!m_shortIdentifiers[characters[0]].isNull())
             return m_shortIdentifiers[characters[0]];
-        m_shortIdentifiers[characters[0]] = Identifier::fromString(&m_exec->vm(), characters, length);
+        m_shortIdentifiers[characters[0]] = Identifier::fromString(vm, characters, length);
         return m_shortIdentifiers[characters[0]];
     }
     if (!m_recentIdentifiers[characters[0]].isNull() && Identifier::equal(m_recentIdentifiers[characters[0]].impl(), characters, length))
         return m_recentIdentifiers[characters[0]];
-    m_recentIdentifiers[characters[0]] = Identifier::fromString(&m_exec->vm(), characters, length);
+    m_recentIdentifiers[characters[0]] = Identifier::fromString(vm, characters, length);
     return m_recentIdentifiers[characters[0]];
 }
 
@@ -452,12 +455,12 @@ ALWAYS_INLINE TokenType LiteralParser<CharType>::Lexer::lex(LiteralParserToken<C
     token.type = TokError;
     token.start = m_ptr;
     CharType character = *m_ptr;
-    if (LIKELY(character < 256)) {
+    if (LIKELY(isLatin1(character))) {
         TokenType tokenType = TokenTypesOfLatin1Characters[character];
         switch (tokenType) {
         case TokString:
             if (character == '\'' && m_mode == StrictJSON) {
-                m_lexErrorMessage = ASCIILiteral("Single quotes (\') are not allowed in JSON");
+                m_lexErrorMessage = "Single quotes (\') are not allowed in JSON"_s;
                 return TokError;
             }
             return lexString(token, character);
@@ -515,7 +518,7 @@ ALWAYS_INLINE TokenType LiteralParser<CharType>::Lexer::lex(LiteralParserToken<C
             return tokenType;
         }
     }
-    m_lexErrorMessage = String::format("Unrecognized token '%c'", *m_ptr);
+    m_lexErrorMessage = makeString("Unrecognized token '", StringView { m_ptr, 1 }, '\'');
     return TokError;
 }
 
@@ -578,7 +581,7 @@ static ALWAYS_INLINE bool isSafeStringCharacter(LChar c, LChar terminator)
 template <SafeStringCharacterSet set>
 static ALWAYS_INLINE bool isSafeStringCharacter(UChar c, UChar terminator)
 {
-    return (c >= ' ' && (set == SafeStringCharacterSet::Strict || c <= 0xff) && c != '\\' && c != terminator) || (c == '\t' && set != SafeStringCharacterSet::Strict);
+    return (c >= ' ' && (set == SafeStringCharacterSet::Strict || isLatin1(c)) && c != '\\' && c != terminator) || (c == '\t' && set != SafeStringCharacterSet::Strict);
 }
 
 template <typename CharType>
@@ -621,15 +624,15 @@ TokenType LiteralParser<CharType>::Lexer::lexStringSlow(LiteralParserToken<CharT
         }
 
         if (!m_builder.isEmpty())
-            m_builder.append(runStart, m_ptr - runStart);
+            m_builder.appendCharacters(runStart, m_ptr - runStart);
 
 slowPathBegin:
         if ((m_mode != NonStrictJSON) && m_ptr < m_end && *m_ptr == '\\') {
             if (m_builder.isEmpty() && runStart < m_ptr)
-                m_builder.append(runStart, m_ptr - runStart);
+                m_builder.appendCharacters(runStart, m_ptr - runStart);
             ++m_ptr;
             if (m_ptr >= m_end) {
-                m_lexErrorMessage = ASCIILiteral("Unterminated string");
+                m_lexErrorMessage = "Unterminated string"_s;
                 return TokError;
             }
             switch (*m_ptr) {
@@ -668,12 +671,12 @@ slowPathBegin:
 
                 case 'u':
                     if ((m_end - m_ptr) < 5) { 
-                        m_lexErrorMessage = ASCIILiteral("\\u must be followed by 4 hex digits");
+                        m_lexErrorMessage = "\\u must be followed by 4 hex digits"_s;
                         return TokError;
                     } // uNNNN == 5 characters
                     for (int i = 1; i < 5; i++) {
                         if (!isASCIIHexDigit(m_ptr[i])) {
-                            m_lexErrorMessage = String::format("\"\\%s\" is not a valid unicode escape", String(m_ptr, 5).ascii().data());
+                            m_lexErrorMessage = makeString("\"\\", StringView { m_ptr, 5 }, "\" is not a valid unicode escape");
                             return TokError;
                         }
                     }
@@ -687,14 +690,14 @@ slowPathBegin:
                         m_ptr++;
                         break;
                     }
-                    m_lexErrorMessage = String::format("Invalid escape character %c", *m_ptr);
+                    m_lexErrorMessage = makeString("Invalid escape character ", StringView { m_ptr, 1 });
                     return TokError;
             }
         }
     } while ((m_mode != NonStrictJSON) && m_ptr != runStart && (m_ptr < m_end) && *m_ptr != terminator);
 
     if (m_ptr >= m_end || *m_ptr != terminator) {
-        m_lexErrorMessage = ASCIILiteral("Unterminated string");
+        m_lexErrorMessage = "Unterminated string"_s;
         return TokError;
     }
 
@@ -745,7 +748,7 @@ TokenType LiteralParser<CharType>::Lexer::lexNumber(LiteralParserToken<CharType>
         while (m_ptr < m_end && isASCIIDigit(*m_ptr))
             ++m_ptr;
     } else {
-        m_lexErrorMessage = ASCIILiteral("Invalid number");
+        m_lexErrorMessage = "Invalid number"_s;
         return TokError;
     }
 
@@ -755,7 +758,7 @@ TokenType LiteralParser<CharType>::Lexer::lexNumber(LiteralParserToken<CharType>
         ++m_ptr;
         // [0-9]+
         if (m_ptr >= m_end || !isASCIIDigit(*m_ptr)) {
-            m_lexErrorMessage = ASCIILiteral("Invalid digits after decimal point");
+            m_lexErrorMessage = "Invalid digits after decimal point"_s;
             return TokError;
         }
 
@@ -798,7 +801,7 @@ TokenType LiteralParser<CharType>::Lexer::lexNumber(LiteralParserToken<CharType>
 
         // [0-9]+
         if (m_ptr >= m_end || !isASCIIDigit(*m_ptr)) {
-            m_lexErrorMessage = ASCIILiteral("Exponent symbols should be followed by an optional '+' or '-' and then by at least one number");
+            m_lexErrorMessage = "Exponent symbols should be followed by an optional '+' or '-' and then by at least one number"_s;
             return TokError;
         }
         
@@ -839,7 +842,7 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
                 TokenType lastToken = m_lexer.currentToken()->type;
                 if (m_lexer.next() == TokRBracket) {
                     if (lastToken == TokComma) {
-                        m_parseErrorMessage = ASCIILiteral("Unexpected comma at the end of array expression");
+                        m_parseErrorMessage = "Unexpected comma at the end of array expression"_s;
                         return JSValue();
                     }
                     m_lexer.next();
@@ -859,7 +862,7 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
                     goto doParseArrayStartExpression;
 
                 if (m_lexer.currentToken()->type != TokRBracket) {
-                    m_parseErrorMessage = ASCIILiteral("Expected ']'");
+                    m_parseErrorMessage = "Expected ']'"_s;
                     return JSValue();
                 }
                 
@@ -882,7 +885,7 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
 
                     // Check for colon
                     if (m_lexer.next() != TokColon) {
-                        m_parseErrorMessage = ASCIILiteral("Expected ':' before value in object property definition");
+                        m_parseErrorMessage = "Expected ':' before value in object property definition"_s;
                         return JSValue();
                     }
                     
@@ -891,7 +894,7 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
                     goto startParseExpression;
                 }
                 if (type != TokRBrace)  {
-                    m_parseErrorMessage = ASCIILiteral("Expected '}'");
+                    m_parseErrorMessage = "Expected '}'"_s;
                     return JSValue();
                 }
                 m_lexer.next();
@@ -902,7 +905,7 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
             case DoParseObjectStartExpression: {
                 TokenType type = m_lexer.next();
                 if (type != TokString && (m_mode == StrictJSON || type != TokIdentifier)) {
-                    m_parseErrorMessage = ASCIILiteral("Property name must be a string literal");
+                    m_parseErrorMessage = "Property name must be a string literal"_s;
                     return JSValue();
                 }
                 typename Lexer::LiteralParserTokenPtr identifierToken = m_lexer.currentToken();
@@ -913,7 +916,7 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
 
                 // Check for colon
                 if (m_lexer.next() != TokColon) {
-                    m_parseErrorMessage = ASCIILiteral("Expected ':'");
+                    m_parseErrorMessage = "Expected ':'"_s;
                     return JSValue();
                 }
 
@@ -927,14 +930,14 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
                 Identifier ident = identifierStack.takeLast();
                 if (m_mode != StrictJSON && ident == vm.propertyNames->underscoreProto) {
                     if (!visitedUnderscoreProto.add(object).isNewEntry) {
-                        m_parseErrorMessage = ASCIILiteral("Attempted to redefine __proto__ property");
+                        m_parseErrorMessage = "Attempted to redefine __proto__ property"_s;
                         return JSValue();
                     }
                     CodeBlock* codeBlock = m_exec->codeBlock();
                     PutPropertySlot slot(object, codeBlock ? codeBlock->isStrictMode() : false);
                     objectStack.last().put(m_exec, ident, lastValue, slot);
                 } else {
-                    if (std::optional<uint32_t> index = parseIndex(ident))
+                    if (Optional<uint32_t> index = parseIndex(ident))
                         object->putDirectIndex(m_exec, index.value(), lastValue);
                     else
                         object->putDirect(vm, ident, lastValue);
@@ -943,7 +946,7 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
                 if (m_lexer.currentToken()->type == TokComma)
                     goto doParseObjectStartExpression;
                 if (m_lexer.currentToken()->type != TokRBrace) {
-                    m_parseErrorMessage = ASCIILiteral("Expected '}'");
+                    m_parseErrorMessage = "Expected '}'"_s;
                     return JSValue();
                 }
                 m_lexer.next();
@@ -960,9 +963,9 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
                     case TokString: {
                         typename Lexer::LiteralParserTokenPtr stringToken = m_lexer.currentToken();
                         if (stringToken->stringIs8Bit)
-                            lastValue = jsString(m_exec, makeIdentifier(stringToken->stringToken8, stringToken->stringLength).string());
+                            lastValue = jsString(vm, makeIdentifier(stringToken->stringToken8, stringToken->stringLength).string());
                         else
-                            lastValue = jsString(m_exec, makeIdentifier(stringToken->stringToken16, stringToken->stringLength).string());
+                            lastValue = jsString(vm, makeIdentifier(stringToken->stringToken16, stringToken->stringLength).string());
                         m_lexer.next();
                         break;
                     }
@@ -987,47 +990,47 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
                         lastValue = jsBoolean(false);
                         break;
                     case TokRBracket:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ']'");
+                        m_parseErrorMessage = "Unexpected token ']'"_s;
                         return JSValue();
                     case TokRBrace:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token '}'");
+                        m_parseErrorMessage = "Unexpected token '}'"_s;
                         return JSValue();
                     case TokIdentifier: {
                         typename Lexer::LiteralParserTokenPtr token = m_lexer.currentToken();
                         if (token->stringIs8Bit)
-                            m_parseErrorMessage = String::format("Unexpected identifier \"%s\"", String(token->stringToken8, token->stringLength).ascii().data());
+                            m_parseErrorMessage = makeString("Unexpected identifier \"", StringView { token->stringToken8, token->stringLength }, '"');
                         else
-                            m_parseErrorMessage = String::format("Unexpected identifier \"%s\"", String(token->stringToken16, token->stringLength).ascii().data());
+                            m_parseErrorMessage = makeString("Unexpected identifier \"", StringView { token->stringToken16, token->stringLength }, '"');
                         return JSValue();
                     }
                     case TokColon:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ':'");
+                        m_parseErrorMessage = "Unexpected token ':'"_s;
                         return JSValue();
                     case TokLParen:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token '('");
+                        m_parseErrorMessage = "Unexpected token '('"_s;
                         return JSValue();
                     case TokRParen:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ')'");
+                        m_parseErrorMessage = "Unexpected token ')'"_s;
                         return JSValue();
                     case TokComma:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ','");
+                        m_parseErrorMessage = "Unexpected token ','"_s;
                         return JSValue();
                     case TokDot:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token '.'");
+                        m_parseErrorMessage = "Unexpected token '.'"_s;
                         return JSValue();
                     case TokAssign:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token '='");
+                        m_parseErrorMessage = "Unexpected token '='"_s;
                         return JSValue();
                     case TokSemi:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ';'");
+                        m_parseErrorMessage = "Unexpected token ';'"_s;
                         return JSValue();
                     case TokEnd:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected EOF");
+                        m_parseErrorMessage = "Unexpected EOF"_s;
                         return JSValue();
                     case TokError:
                     default:
                         // Error
-                        m_parseErrorMessage = ASCIILiteral("Could not parse value expression");
+                        m_parseErrorMessage = "Could not parse value expression"_s;
                         return JSValue();
                 }
                 break;
@@ -1045,50 +1048,50 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
                         goto startParseExpression;
                     }
                     case TokRBracket:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ']'");
+                        m_parseErrorMessage = "Unexpected token ']'"_s;
                         return JSValue();
                     case TokLBrace:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token '{'");
+                        m_parseErrorMessage = "Unexpected token '{'"_s;
                         return JSValue();
                     case TokRBrace:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token '}'");
+                        m_parseErrorMessage = "Unexpected token '}'"_s;
                         return JSValue();
                     case TokIdentifier:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected identifier");
+                        m_parseErrorMessage = "Unexpected identifier"_s;
                         return JSValue();
                     case TokColon:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ':'");
+                        m_parseErrorMessage = "Unexpected token ':'"_s;
                         return JSValue();
                     case TokRParen:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ')'");
+                        m_parseErrorMessage = "Unexpected token ')'"_s;
                         return JSValue();
                     case TokComma:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ','");
+                        m_parseErrorMessage = "Unexpected token ','"_s;
                         return JSValue();
                     case TokTrue:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token 'true'");
+                        m_parseErrorMessage = "Unexpected token 'true'"_s;
                         return JSValue();
                     case TokFalse:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token 'false'");
+                        m_parseErrorMessage = "Unexpected token 'false'"_s;
                         return JSValue();
                     case TokNull:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token 'null'");
+                        m_parseErrorMessage = "Unexpected token 'null'"_s;
                         return JSValue();
                     case TokEnd:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected EOF");
+                        m_parseErrorMessage = "Unexpected EOF"_s;
                         return JSValue();
                     case TokDot:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token '.'");
+                        m_parseErrorMessage = "Unexpected token '.'"_s;
                         return JSValue();
                     case TokAssign:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token '='");
+                        m_parseErrorMessage = "Unexpected token '='"_s;
                         return JSValue();
                     case TokSemi:
-                        m_parseErrorMessage = ASCIILiteral("Unexpected token ';'");
+                        m_parseErrorMessage = "Unexpected token ';'"_s;
                         return JSValue();
                     case TokError:
                     default:
-                        m_parseErrorMessage = ASCIILiteral("Could not parse statement");
+                        m_parseErrorMessage = "Could not parse statement"_s;
                         return JSValue();
                 }
             }
@@ -1098,7 +1101,7 @@ JSValue LiteralParser<CharType>::parse(ParserState initialState)
                     return JSValue();
                 if (m_lexer.next() == TokEnd)
                     return lastValue;
-                m_parseErrorMessage = ASCIILiteral("Unexpected content at end of JSON literal");
+                m_parseErrorMessage = "Unexpected content at end of JSON literal"_s;
                 return JSValue();
             }
             default:
