@@ -26,7 +26,7 @@
 #include "config.h"
 #include "WebMediaSessionManager.h"
 
-#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS)
+#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
 
 #include "FloatRect.h"
 #include "Logging.h"
@@ -39,6 +39,8 @@ namespace WebCore {
 static const Seconds taskDelayInterval { 100_ms };
 
 struct ClientState {
+    WTF_MAKE_STRUCT_FAST_ALLOCATED;
+
     explicit ClientState(WebMediaSessionManagerClient& client, uint64_t contextId)
         : client(client)
         , contextId(contextId)
@@ -113,7 +115,7 @@ void WebMediaSessionManager::setMockMediaPlaybackTargetPickerState(const String&
 MediaPlaybackTargetPickerMock& WebMediaSessionManager::mockPicker()
 {
     if (!m_pickerOverride)
-        m_pickerOverride = std::make_unique<MediaPlaybackTargetPickerMock>(*this);
+        m_pickerOverride = makeUnique<MediaPlaybackTargetPickerMock>(*this);
 
     return *m_pickerOverride.get();
 }
@@ -143,7 +145,7 @@ uint64_t WebMediaSessionManager::addPlaybackTargetPickerClient(WebMediaSessionMa
 
     LOG(Media, "WebMediaSessionManager::addPlaybackTargetPickerClient(%p + %llu)", &client, contextId);
 
-    m_clientState.append(std::make_unique<ClientState>(client, contextId));
+    m_clientState.append(makeUnique<ClientState>(client, contextId));
 
     if (m_externalOutputDeviceAvailable || m_playbackTarget)
         scheduleDelayedTask(InitialConfigurationTask | TargetClientsConfigurationTask);
@@ -464,4 +466,4 @@ void WebMediaSessionManager::watchdogTimerFired()
 
 } // namespace WebCore
 
-#endif // ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS)
+#endif // ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
