@@ -27,9 +27,9 @@
 #include "config.h"
 #include "CSSFilterImageValue.h"
 
+#include "CSSFilter.h"
 #include "CachedImage.h"
 #include "CachedResourceLoader.h"
-#include "FilterEffectRenderer.h"
 #include "GraphicsContext.h"
 #include "ImageBuffer.h"
 #include "RenderElement.h"
@@ -47,13 +47,7 @@ CSSFilterImageValue::~CSSFilterImageValue()
 
 String CSSFilterImageValue::customCSSText() const
 {
-    StringBuilder result;
-    result.appendLiteral("filter(");
-    result.append(m_imageValue->cssText());
-    result.appendLiteral(", ");
-    result.append(m_filterValue->cssText());
-    result.append(')');
-    return result.toString();
+    return makeString("filter(", m_imageValue->cssText(), ", ", m_filterValue->cssText(), ')');
 }
 
 FloatSize CSSFilterImageValue::fixedSize(const RenderElement* renderer)
@@ -131,15 +125,15 @@ RefPtr<Image> CSSFilterImageValue::image(RenderElement* renderer, const FloatSiz
     auto imageRect = FloatRect { { }, size };
     texture->context().drawImage(*image, imageRect);
 
-    auto filterRenderer = FilterEffectRenderer::create();
-    filterRenderer->setSourceImage(WTFMove(texture));
-    filterRenderer->setSourceImageRect(imageRect);
-    filterRenderer->setFilterRegion(imageRect);
-    if (!filterRenderer->build(*renderer, m_filterOperations, FilterFunction))
+    auto cssFilter = CSSFilter::create();
+    cssFilter->setSourceImage(WTFMove(texture));
+    cssFilter->setSourceImageRect(imageRect);
+    cssFilter->setFilterRegion(imageRect);
+    if (!cssFilter->build(*renderer, m_filterOperations, FilterConsumer::FilterFunction))
         return &Image::nullImage();
-    filterRenderer->apply();
+    cssFilter->apply();
 
-    return filterRenderer->output()->copyImage();
+    return cssFilter->output()->copyImage();
 }
 
 void CSSFilterImageValue::filterImageChanged(const IntRect&)
