@@ -26,15 +26,14 @@
 #include "config.h"
 #include "InsertListCommand.h"
 
+#include "Editing.h"
 #include "ElementTraversal.h"
-#include "ExceptionCodePlaceholder.h"
 #include "HTMLBRElement.h"
 #include "HTMLLIElement.h"
 #include "HTMLNames.h"
 #include "HTMLUListElement.h"
 #include "Range.h"
 #include "VisibleUnits.h"
-#include "htmlediting.h"
 
 namespace WebCore {
 
@@ -55,35 +54,33 @@ RefPtr<HTMLElement> InsertListCommand::insertList(Document& document, Type type)
     return insertCommand->m_listElement;
 }
 
-HTMLElement* InsertListCommand::fixOrphanedListChild(Node* node)
+HTMLElement& InsertListCommand::fixOrphanedListChild(Node& node)
 {
     auto listElement = HTMLUListElement::create(document());
     insertNodeBefore(listElement.copyRef(), node);
     removeNode(node);
     appendNode(node, listElement.copyRef());
-    m_listElement = listElement.copyRef();
-    return listElement.ptr();
+    m_listElement = WTFMove(listElement);
+    return *m_listElement;
 }
 
-RefPtr<HTMLElement> InsertListCommand::mergeWithNeighboringLists(PassRefPtr<HTMLElement> passedList)
+Ref<HTMLElement> InsertListCommand::mergeWithNeighboringLists(HTMLElement& list)
 {
-    RefPtr<HTMLElement> list = passedList;
-    Element* previousList = list->previousElementSibling();
-    if (canMergeLists(previousList, list.get()))
-        mergeIdenticalElements(previousList, list);
+    Ref<HTMLElement> protectedList = list;
+    Element* previousList = list.previousElementSibling();
+    if (canMergeLists(previousList, &list))
+        mergeIdenticalElements(*previousList, list);
 
-    if (!list)
-        return nullptr;
-    Element* sibling = ElementTraversal::nextSibling(*list);
+    Element* sibling = ElementTraversal::nextSibling(list);
     if (!is<HTMLElement>(sibling))
-        return list;
+        return protectedList;
 
-    RefPtr<HTMLElement> nextList = downcast<HTMLElement>(sibling);
-    if (canMergeLists(list.get(), nextList.get())) {
+    Ref<HTMLElement> nextList = downcast<HTMLElement>(*sibling);
+    if (canMergeLists(&list, nextList.ptr())) {
         mergeIdenticalElements(list, nextList);
         return nextList;
     }
-    return list;
+    return protectedList;
 }
 
 bool InsertListCommand::selectionHasListOfType(const VisibleSelection& selection, const QualifiedName& listTag)
@@ -131,7 +128,7 @@ void InsertListCommand::doApply()
             return;
     }
 
-    auto& listTag = (m_type == OrderedList) ? olTag : ulTag;
+    auto& listTag = (m_type == Type::OrderedList) ? olTag : ulTag;
     if (endingSelection().isRange()) {
         VisibleSelection selection = selectionForParagraphIteration(endingSelection());
         ASSERT(selection.isRange());
@@ -150,7 +147,7 @@ void InsertListCommand::doApply()
                 // infinite loop and because there is no more work to be done.
                 // FIXME(<rdar://problem/5983974>): The endingSelection() may be incorrect here.  Compute 
                 // the new location of endOfSelection and use it as the end of the new selection.
-                if (!startOfLastParagraph.deepEquivalent().anchorNode()->inDocument())
+                if (!startOfLastParagraph.deepEquivalent().anchorNode()->isConnected())
                     return;
                 setEndingSelection(startOfCurrentParagraph);
 
@@ -195,7 +192,7 @@ void InsertListCommand::doApply()
 
 EditAction InsertListCommand::editingAction() const
 {
-    return m_type == OrderedList ? EditActionInsertOrderedList : EditActionInsertUnorderedList;
+    return m_type == Type::OrderedList ? EditAction::InsertOrderedList : EditAction::InsertUnorderedList;
 }
 
 void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HTMLQualifiedName& listTag, Range* currentSelection)
@@ -209,10 +206,9 @@ void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HT
     if (listChildNode) {
         // Remove the list chlild.
         RefPtr<HTMLElement> listNode = enclosingList(listChildNode);
-        if (!listNode) {
-            listNode = fixOrphanedListChild(listChildNode);
-            listNode = mergeWithNeighboringLists(listNode);
-        }
+        if (!listNode)
+            listNode = mergeWithNeighboringLists(fixOrphanedListChild(*listChildNode));
+
         if (!listNode->hasTagName(listTag)) {
             // listChildNode will be removed from the list and a list of type m_type will be created.
             switchListType = true;
@@ -228,7 +224,7 @@ void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HT
             bool rangeEndIsInList = visiblePositionAfterNode(*listNode) == currentSelection->endPosition();
 
             RefPtr<HTMLElement> newList = createHTMLElement(document(), listTag);
-            insertNodeBefore(newList, listNode);
+            insertNodeBefore(*newList, *listNode);
 
             auto* firstChildInList = enclosingListChild(VisiblePosition(firstPositionInNode(listNode.get())).deepEquivalent().deprecatedNode(), listNode.get());
             Node* outerBlock = firstChildInList && isBlockFlowElement(*firstChildInList) ? firstChildInList : listNode.get();
@@ -238,17 +234,17 @@ void InsertListCommand::doApplyForSingleParagraph(bool forceCreateList, const HT
             // Manually remove listNode because moveParagraphWithClones sometimes leaves it behind in the document.
             // See the bug 33668 and editing/execCommand/insert-list-orphaned-item-with-nested-lists.html.
             // FIXME: This might be a bug in moveParagraphWithClones or deleteSelection.
-            if (listNode && listNode->inDocument())
-                removeNode(listNode);
+            if (listNode && listNode->isConnected())
+                removeNode(*listNode);
 
-            newList = mergeWithNeighboringLists(newList);
+            newList = mergeWithNeighboringLists(*newList);
 
             // Restore the start and the end of current selection if they started inside listNode
             // because moveParagraphWithClones could have removed them.
             if (rangeStartIsInList && newList)
-                currentSelection->setStart(*newList, 0, IGNORE_EXCEPTION);
+                currentSelection->setStart(*newList, 0);
             if (rangeEndIsInList && newList)
-                currentSelection->setEnd(*newList, lastOffsetInNode(newList.get()), IGNORE_EXCEPTION);
+                currentSelection->setEnd(*newList, lastOffsetInNode(newList.get()));
 
             setEndingSelection(VisiblePosition(firstPositionInNode(newList.get())));
 
@@ -284,13 +280,13 @@ void InsertListCommand::unlistifyParagraph(const VisiblePosition& originalStart,
     }
     // When removing a list, we must always create a placeholder to act as a point of insertion
     // for the list content being removed.
-    RefPtr<Element> placeholder = HTMLBRElement::create(document());
-    RefPtr<Element> nodeToInsert = placeholder;
+    auto placeholder = HTMLBRElement::create(document());
+    RefPtr<Element> nodeToInsert = placeholder.copyRef();
     // If the content of the list item will be moved into another list, put it in a list item
     // so that we don't create an orphaned list child.
     if (enclosingList(listNode)) {
         nodeToInsert = HTMLLIElement::create(document());
-        appendNode(placeholder, nodeToInsert);
+        appendNode(placeholder.copyRef(), *nodeToInsert);
     }
 
     if (nextListChild && previousListChild) {
@@ -301,20 +297,20 @@ void InsertListCommand::unlistifyParagraph(const VisiblePosition& originalStart,
         // FIXME: We appear to split at nextListChild as opposed to listChildNode so that when we remove
         // listChildNode below in moveParagraphs, previousListChild will be removed along with it if it is 
         // unrendered. But we ought to remove nextListChild too, if it is unrendered.
-        splitElement(listNode, splitTreeToNode(nextListChild, listNode));
-        insertNodeBefore(nodeToInsert, listNode);
+        splitElement(*listNode, *splitTreeToNode(*nextListChild, *listNode));
+        insertNodeBefore(nodeToInsert.releaseNonNull(), *listNode);
     } else if (nextListChild || listChildNode->parentNode() != listNode) {
         // Just because listChildNode has no previousListChild doesn't mean there isn't any content
         // in listNode that comes before listChildNode, as listChildNode could have ancestors
         // between it and listNode. So, we split up to listNode before inserting the placeholder
         // where we're about to move listChildNode to.
         if (listChildNode->parentNode() != listNode)
-            splitElement(listNode, splitTreeToNode(listChildNode, listNode).get());
-        insertNodeBefore(nodeToInsert, listNode);
+            splitElement(*listNode, *splitTreeToNode(*listChildNode, *listNode).get());
+        insertNodeBefore(nodeToInsert.releaseNonNull(), *listNode);
     } else
-        insertNodeAfter(nodeToInsert, listNode);
+        insertNodeAfter(nodeToInsert.releaseNonNull(), *listNode);
 
-    VisiblePosition insertionPoint = VisiblePosition(positionBeforeNode(placeholder.get()));
+    VisiblePosition insertionPoint = VisiblePosition(positionBeforeNode(placeholder.ptr()));
     moveParagraphs(start, end, insertionPoint, true);
 }
 
@@ -355,13 +351,13 @@ RefPtr<HTMLElement> InsertListCommand::listifyParagraph(const VisiblePosition& o
     Element* nextList = adjacentEnclosingList(start.deepEquivalent(), end.next(CannotCrossEditingBoundary), listTag);
     RefPtr<HTMLElement> listElement;
     if (previousList)
-        appendNode(WTFMove(listItemElement), previousList);
+        appendNode(WTFMove(listItemElement), *previousList);
     else if (nextList)
         insertNodeAt(WTFMove(listItemElement), positionBeforeNode(nextList));
     else {
         // Create the list.
         listElement = createHTMLElement(document(), listTag);
-        appendNode(WTFMove(listItemElement), listElement);
+        appendNode(WTFMove(listItemElement), *listElement);
 
         if (start == end && isBlock(start.deepEquivalent().deprecatedNode())) {
             // Inserting the list into an empty paragraph that isn't held open 
@@ -383,7 +379,7 @@ RefPtr<HTMLElement> InsertListCommand::listifyParagraph(const VisiblePosition& o
         if (listChild && listChild->hasTagName(liTag))
             insertionPos = positionInParentBeforeNode(listChild);
 
-        insertNodeAt(listElement, insertionPos);
+        insertNodeAt(*listElement, insertionPos);
 
         // We inserted the list at the start of the content we're about to move
         // Update the start of content, so we don't try to move the list into itself.  bug 19066
@@ -399,10 +395,10 @@ RefPtr<HTMLElement> InsertListCommand::listifyParagraph(const VisiblePosition& o
     moveParagraph(start, end, positionBeforeNode(placeholder.ptr()), true);
 
     if (listElement)
-        return mergeWithNeighboringLists(listElement);
+        return mergeWithNeighboringLists(*listElement);
 
     if (canMergeLists(previousList, nextList))
-        mergeIdenticalElements(previousList, nextList);
+        mergeIdenticalElements(*previousList, *nextList);
 
     return listElement;
 }
