@@ -27,13 +27,13 @@
 #include "Chrome.h"
 #include "ChromeClient.h"
 #include "DOMFormData.h"
-#include "Document.h"
 #include "File.h"
-#include "FileSystem.h"
 #include "FormDataBuilder.h"
 #include "Page.h"
+#include "SharedBuffer.h"
 #include "TextEncoding.h"
 #include "ThreadableBlobRegistry.h"
+#include <wtf/FileSystem.h>
 #include <wtf/text/LineEnding.h>
 
 namespace WebCore {
@@ -49,22 +49,10 @@ inline FormData::FormData(const FormData& data)
     , m_alwaysStream(false)
     , m_containsPasswordData(data.m_containsPasswordData)
 {
-    // We shouldn't be copying FormData that hasn't already removed its generated files
-    // but just in case, make sure the new FormData is ready to generate its own files.
-    for (auto& element : m_elements) {
-        if (element.m_type == FormDataElement::Type::EncodedFile) {
-            element.m_generatedFilename = String();
-            element.m_ownsGeneratedFile = false;
-        }
-    }
 }
 
 FormData::~FormData()
 {
-    // This cleanup should've happened when the form submission finished.
-    // Just in case, let's assert, and do the cleanup anyway in release builds.
-    ASSERT(!hasOwnedGeneratedFiles());
-    removeGeneratedFilesIfNeeded();
 }
 
 Ref<FormData> FormData::create()
@@ -89,6 +77,13 @@ Ref<FormData> FormData::create(const Vector<char>& vector)
     return create(vector.data(), vector.size());
 }
 
+Ref<FormData> FormData::create(Vector<char>&& vector)
+{
+    auto data = create();
+    data->m_elements.append(WTFMove(vector));
+    return data;
+}
+
 Ref<FormData> FormData::create(const Vector<uint8_t>& vector)
 {
     return create(vector.data(), vector.size());
@@ -101,10 +96,10 @@ Ref<FormData> FormData::create(const DOMFormData& formData, EncodingType encodin
     return result;
 }
 
-Ref<FormData> FormData::createMultiPart(const DOMFormData& formData, Document* document)
+Ref<FormData> FormData::createMultiPart(const DOMFormData& formData)
 {
     auto result = create();
-    result->appendMultiPartKeyValuePairItems(formData, document);
+    result->appendMultiPartKeyValuePairItems(formData);
     return result;
 }
 
@@ -129,61 +124,83 @@ Ref<FormData> FormData::isolatedCopy() const
     return formData;
 }
 
-uint64_t FormDataElement::lengthInBytes() const
+static inline uint64_t computeLengthInBytes(const FormDataElement& element, const Function<uint64_t(const URL&)>& blobSize)
 {
-    switch (m_type) {
-    case Type::Data:
-        return m_data.size();
-    case Type::EncodedFile: {
-        if (m_fileLength != BlobDataItem::toEndOfFile)
-            return m_fileLength;
-        long long fileSize;
-        if (FileSystem::getFileSize(m_shouldGenerateFile ? m_generatedFilename : m_filename, fileSize))
-            return fileSize;
-        return 0;
-    }
-    case Type::EncodedBlob:
-        return ThreadableBlobRegistry::blobSize(m_url);
-    }
-    ASSERT_NOT_REACHED();
-    return 0;
+    return switchOn(element.data,
+        [] (const Vector<char>& bytes) {
+            return static_cast<uint64_t>(bytes.size());
+        }, [] (const FormDataElement::EncodedFileData& fileData) {
+            if (fileData.fileLength != BlobDataItem::toEndOfFile)
+                return static_cast<uint64_t>(fileData.fileLength);
+            long long fileSize;
+            if (FileSystem::getFileSize(fileData.filename, fileSize))
+                return static_cast<uint64_t>(fileSize);
+            return static_cast<uint64_t>(0);
+        }, [&blobSize] (const FormDataElement::EncodedBlobData& blobData) {
+            return blobSize(blobData.url);
+        }
+    );
+}
+
+uint64_t FormDataElement::lengthInBytes(BlobRegistryImpl* blobRegistry) const
+{
+    return computeLengthInBytes(*this, [&](auto& url) {
+        return blobRegistry ? blobRegistry->blobSize(url) : 0;
+    });
+}
+
+uint64_t FormDataElement::lengthInBytes(PAL::SessionID sessionID) const
+{
+    return computeLengthInBytes(*this, [&](auto& url) {
+        return blobRegistry().blobSize(sessionID, url);
+    });
 }
 
 FormDataElement FormDataElement::isolatedCopy() const
 {
-    switch (m_type) {
-    case Type::Data:
-        return FormDataElement(m_data);
-    case Type::EncodedFile:
-        return FormDataElement(m_filename.isolatedCopy(), m_fileStart, m_fileLength, m_expectedFileModificationTime, m_shouldGenerateFile);
-    case Type::EncodedBlob:
-        return FormDataElement(m_url.isolatedCopy());
-    }
-
-    RELEASE_ASSERT_NOT_REACHED();
+    return switchOn(data,
+        [] (const Vector<char>& bytes) {
+            Vector<char> copy;
+            copy.append(bytes.data(), bytes.size());
+            return FormDataElement(WTFMove(copy));
+        }, [] (const FormDataElement::EncodedFileData& fileData) {
+            return FormDataElement(fileData.isolatedCopy());
+        }, [] (const FormDataElement::EncodedBlobData& blobData) {
+            return FormDataElement(blobData.url.isolatedCopy());
+        }
+    );
 }
 
 void FormData::appendData(const void* data, size_t size)
 {
-    memcpy(expandDataStore(size), data, size);
+    m_lengthInBytes = WTF::nullopt;
+    if (!m_elements.isEmpty()) {
+        if (auto* vector = WTF::get_if<Vector<char>>(m_elements.last().data)) {
+            vector->append(reinterpret_cast<const char*>(data), size);
+            return;
+        }
+    }
+    Vector<char> vector;
+    vector.append(reinterpret_cast<const char*>(data), size);
+    m_elements.append(WTFMove(vector));
 }
 
-void FormData::appendFile(const String& filename, bool shouldGenerateFile)
+void FormData::appendFile(const String& filename)
 {
-    m_elements.append(FormDataElement(filename, 0, BlobDataItem::toEndOfFile, FileSystem::invalidFileTime(), shouldGenerateFile));
-    m_lengthInBytes = std::nullopt;
+    m_elements.append(FormDataElement(filename, 0, BlobDataItem::toEndOfFile, WTF::nullopt));
+    m_lengthInBytes = WTF::nullopt;
 }
 
-void FormData::appendFileRange(const String& filename, long long start, long long length, double expectedModificationTime, bool shouldGenerateFile)
+void FormData::appendFileRange(const String& filename, long long start, long long length, Optional<WallTime> expectedModificationTime)
 {
-    m_elements.append(FormDataElement(filename, start, length, expectedModificationTime, shouldGenerateFile));
-    m_lengthInBytes = std::nullopt;
+    m_elements.append(FormDataElement(filename, start, length, expectedModificationTime));
+    m_lengthInBytes = WTF::nullopt;
 }
 
 void FormData::appendBlob(const URL& blobURL)
 {
     m_elements.append(FormDataElement(blobURL));
-    m_lengthInBytes = std::nullopt;
+    m_lengthInBytes = WTF::nullopt;
 }
 
 static Vector<uint8_t> normalizeStringData(TextEncoding& encoding, const String& value)
@@ -191,22 +208,10 @@ static Vector<uint8_t> normalizeStringData(TextEncoding& encoding, const String&
     return normalizeLineEndingsToCRLF(encoding.encode(value, UnencodableHandling::Entities));
 }
 
-void FormData::appendMultiPartFileValue(const File& file, Vector<char>& header, TextEncoding& encoding, Document* document)
+void FormData::appendMultiPartFileValue(const File& file, Vector<char>& header, TextEncoding& encoding)
 {
     auto name = file.name();
 
-    // Let the application specify a filename if it's going to generate a replacement file for the upload.
-    bool shouldGenerateFile = false;
-    auto& path = file.path();
-    if (!path.isEmpty()) {
-        if (Page* page = document->page()) {
-            String generatedFileName;
-            shouldGenerateFile = page->chrome().client().shouldReplaceWithGeneratedFileForUpload(path, generatedFileName);
-            if (shouldGenerateFile)
-                name = generatedFileName;
-        }
-    }
-
     // We have to include the filename=".." part in the header, even if the filename is empty
     FormDataBuilder::addFilenameToMultiPartHeader(header, encoding, name);
 
@@ -222,7 +227,7 @@ void FormData::appendMultiPartFileValue(const File& file, Vector<char>& header,
     appendData(header.data(), header.size());
 
     if (!file.path().isEmpty())
-        appendFile(file.path(), shouldGenerateFile);
+        appendFile(file.path());
     else if (file.size())
         appendBlob(file.url());
 }
@@ -236,7 +241,7 @@ void FormData::appendMultiPartStringValue(const String& string, Vector<char>& he
     appendData(normalizedStringData.data(), normalizedStringData.size());
 }
 
-void FormData::appendMultiPartKeyValuePairItems(const DOMFormData& formData, Document* document)
+void FormData::appendMultiPartKeyValuePairItems(const DOMFormData& formData)
 {
     m_boundary = FormDataBuilder::generateUniqueBoundaryString();
 
@@ -250,7 +255,7 @@ void FormData::appendMultiPartKeyValuePairItems(const DOMFormData& formData, Doc
         FormDataBuilder::beginMultiPartHeader(header, m_boundary.data(), normalizedName);
 
         if (WTF::holds_alternative<RefPtr<File>>(item.data))
-            appendMultiPartFileValue(*WTF::get<RefPtr<File>>(item.data), header, encoding, document);
+            appendMultiPartFileValue(*WTF::get<RefPtr<File>>(item.data), header, encoding);
         else
             appendMultiPartStringValue(WTF::get<String>(item.data), header, encoding);
 
@@ -278,28 +283,13 @@ void FormData::appendNonMultiPartKeyValuePairItems(const DOMFormData& formData,
     appendData(encodedData.data(), encodedData.size());
 }
 
-char* FormData::expandDataStore(size_t size)
-{
-    m_lengthInBytes = std::nullopt;
-    if (m_elements.isEmpty() || m_elements.last().m_type != FormDataElement::Type::Data)
-        m_elements.append({ });
-
-    auto& lastElement = m_elements.last();
-    size_t oldSize = lastElement.m_data.size();
-
-    auto newSize = Checked<size_t>(oldSize) + size;
-
-    lastElement.m_data.grow(newSize.unsafeGet());
-    return lastElement.m_data.data() + oldSize;
-}
-
 Vector<char> FormData::flatten() const
 {
     // Concatenate all the byte arrays, but omit any files.
     Vector<char> data;
     for (auto& element : m_elements) {
-        if (element.m_type == FormDataElement::Type::Data)
-            data.append(element.m_data.data(), static_cast<size_t>(element.m_data.size()));
+        if (auto* vector = WTF::get_if<Vector<char>>(element.data))
+            data.append(vector->data(), vector->size());
     }
     return data;
 }
@@ -310,14 +300,14 @@ String FormData::flattenToString() const
     return Latin1Encoding().decode(reinterpret_cast<const char*>(bytes.data()), bytes.size());
 }
 
-static void appendBlobResolved(FormData* formData, const URL& url)
+static void appendBlobResolved(BlobRegistryImpl* blobRegistry, FormData& formData, const URL& url)
 {
-    if (!blobRegistry().isBlobRegistryImpl()) {
+    if (!blobRegistry) {
         LOG_ERROR("Tried to resolve a blob without a usable registry");
         return;
     }
 
-    BlobData* blobData = static_cast<BlobRegistryImpl&>(blobRegistry()).getBlobDataFromURL(url);
+    auto* blobData = blobRegistry->getBlobDataFromURL(url);
     if (!blobData) {
         LOG_ERROR("Could not get blob data from a registry");
         return;
@@ -326,20 +316,20 @@ static void appendBlobResolved(FormData* formData, const URL& url)
     for (const auto& blobItem : blobData->items()) {
         if (blobItem.type() == BlobDataItem::Type::Data) {
             ASSERT(blobItem.data().data());
-            formData->appendData(blobItem.data().data()->data() + static_cast<int>(blobItem.offset()), static_cast<int>(blobItem.length()));
+            formData.appendData(blobItem.data().data()->data() + static_cast<int>(blobItem.offset()), static_cast<int>(blobItem.length()));
         } else if (blobItem.type() == BlobDataItem::Type::File)
-            formData->appendFileRange(blobItem.file()->path(), blobItem.offset(), blobItem.length(), blobItem.file()->expectedModificationTime());
+            formData.appendFileRange(blobItem.file()->path(), blobItem.offset(), blobItem.length(), blobItem.file()->expectedModificationTime());
         else
             ASSERT_NOT_REACHED();
     }
 }
 
-Ref<FormData> FormData::resolveBlobReferences()
+Ref<FormData> FormData::resolveBlobReferences(BlobRegistryImpl* blobRegistry)
 {
     // First check if any blobs needs to be resolved, or we can take the fast path.
     bool hasBlob = false;
     for (auto& element : m_elements) {
-        if (element.m_type == FormDataElement::Type::EncodedBlob) {
+        if (WTF::holds_alternative<FormDataElement::EncodedBlobData>(element.data)) {
             hasBlob = true;
             break;
         }
@@ -354,78 +344,62 @@ Ref<FormData> FormData::resolveBlobReferences()
     newFormData->setIdentifier(identifier());
 
     for (auto& element : m_elements) {
-        if (element.m_type == FormDataElement::Type::Data)
-            newFormData->appendData(element.m_data.data(), element.m_data.size());
-        else if (element.m_type == FormDataElement::Type::EncodedFile)
-            newFormData->appendFileRange(element.m_filename, element.m_fileStart, element.m_fileLength, element.m_expectedFileModificationTime, element.m_shouldGenerateFile);
-        else if (element.m_type == FormDataElement::Type::EncodedBlob)
-            appendBlobResolved(newFormData.ptr(), element.m_url);
-        else
-            ASSERT_NOT_REACHED();
+        switchOn(element.data,
+            [&] (const Vector<char>& bytes) {
+                newFormData->appendData(bytes.data(), bytes.size());
+            }, [&] (const FormDataElement::EncodedFileData& fileData) {
+                newFormData->appendFileRange(fileData.filename, fileData.fileStart, fileData.fileLength, fileData.expectedFileModificationTime);
+            }, [&] (const FormDataElement::EncodedBlobData& blobData) {
+                appendBlobResolved(blobRegistry, newFormData.get(), blobData.url);
+            }
+        );
     }
     return newFormData;
 }
 
-void FormData::generateFiles(Document* document)
+FormDataForUpload FormData::prepareForUpload()
 {
-    Page* page = document->page();
-    if (!page)
-        return;
-
+    Vector<String> generatedFiles;
     for (auto& element : m_elements) {
-        if (element.m_type == FormDataElement::Type::EncodedFile && element.m_shouldGenerateFile) {
-            ASSERT(!element.m_ownsGeneratedFile);
-            ASSERT(element.m_generatedFilename.isEmpty());
-            if (!element.m_generatedFilename.isEmpty())
-                continue;
-            element.m_generatedFilename = page->chrome().client().generateReplacementFile(element.m_filename);
-            if (!element.m_generatedFilename.isEmpty())
-                element.m_ownsGeneratedFile = true;
-        }
+        auto* fileData = WTF::get_if<FormDataElement::EncodedFileData>(element.data);
+        if (!fileData)
+            continue;
+        if (!FileSystem::fileIsDirectory(fileData->filename, FileSystem::ShouldFollowSymbolicLinks::Yes))
+            continue;
+        if (fileData->fileStart || fileData->fileLength != BlobDataItem::toEndOfFile)
+            continue;
+        if (!fileData->fileModificationTimeMatchesExpectation())
+            continue;
+
+        auto generatedFilename = FileSystem::createTemporaryZipArchive(fileData->filename);
+        if (!generatedFilename)
+            continue;
+        fileData->filename = generatedFilename;
+        generatedFiles.append(WTFMove(generatedFilename));
     }
+    
+    return { *this, WTFMove(generatedFiles) };
 }
 
-bool FormData::hasGeneratedFiles() const
+FormDataForUpload::FormDataForUpload(FormData& data, Vector<String>&& temporaryZipFiles)
+    : m_data(data)
+    , m_temporaryZipFiles(WTFMove(temporaryZipFiles))
 {
-    for (auto& element : m_elements) {
-        if (element.m_type == FormDataElement::Type::EncodedFile && !element.m_generatedFilename.isEmpty())
-            return true;
-    }
-    return false;
 }
 
-bool FormData::hasOwnedGeneratedFiles() const
+FormDataForUpload::~FormDataForUpload()
 {
-    for (auto& element : m_elements) {
-        if (element.m_type == FormDataElement::Type::EncodedFile && element.m_ownsGeneratedFile) {
-            ASSERT(!element.m_generatedFilename.isEmpty());
-            return true;
-        }
-    }
-    return false;
+    ASSERT(isMainThread());
+    for (auto& file : m_temporaryZipFiles)
+        FileSystem::deleteFile(file);
 }
 
-void FormData::removeGeneratedFilesIfNeeded()
-{
-    for (auto& element : m_elements) {
-        if (element.m_type == FormDataElement::Type::EncodedFile && element.m_ownsGeneratedFile) {
-            ASSERT(!element.m_generatedFilename.isEmpty());
-            ASSERT(element.m_shouldGenerateFile);
-            String directory = FileSystem::directoryName(element.m_generatedFilename);
-            FileSystem::deleteFile(element.m_generatedFilename);
-            FileSystem::deleteEmptyDirectory(directory);
-            element.m_generatedFilename = String();
-            element.m_ownsGeneratedFile = false;
-        }
-    }
-}
-
-uint64_t FormData::lengthInBytes() const
+uint64_t FormData::lengthInBytes(PAL::SessionID sessionID) const
 {
     if (!m_lengthInBytes) {
         uint64_t length = 0;
         for (auto& element : m_elements)
-            length += element.lengthInBytes();
+            length += element.lengthInBytes(sessionID);
         m_lengthInBytes = length;
     }
     return *m_lengthInBytes;
@@ -434,7 +408,7 @@ uint64_t FormData::lengthInBytes() const
 RefPtr<SharedBuffer> FormData::asSharedBuffer() const
 {
     for (auto& element : m_elements) {
-        if (element.m_type != FormDataElement::Type::Data)
+        if (!WTF::holds_alternative<Vector<char>>(element.data))
             return nullptr;
     }
     return SharedBuffer::create(flatten());
@@ -445,8 +419,24 @@ URL FormData::asBlobURL() const
     if (m_elements.size() != 1)
         return { };
 
-    ASSERT(m_elements.first().m_type == FormDataElement::Type::EncodedBlob || m_elements.first().m_url.isNull());
-    return m_elements.first().m_url;
+    if (auto* blobData = WTF::get_if<FormDataElement::EncodedBlobData>(m_elements.first().data))
+        return blobData->url;
+    return { };
+}
+
+bool FormDataElement::EncodedFileData::fileModificationTimeMatchesExpectation() const
+{
+    if (!expectedFileModificationTime)
+        return true;
+
+    auto fileModificationTime = FileSystem::getFileModificationTime(filename);
+    if (!fileModificationTime)
+        return false;
+
+    if (fileModificationTime->secondsSinceEpoch().secondsAs<time_t>() != expectedFileModificationTime->secondsSinceEpoch().secondsAs<time_t>())
+        return false;
+
+    return true;
 }
 
 } // namespace WebCore
