@@ -18,8 +18,7 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef SVGFEFuncGElement_h
-#define SVGFEFuncGElement_h
+#pragma once
 
 #include "SVGComponentTransferFunctionElement.h"
 
@@ -27,12 +26,10 @@ namespace WebCore {
 
 class SVGFEFuncGElement final : public SVGComponentTransferFunctionElement {
 public:
-    static PassRefPtr<SVGFEFuncGElement> create(const QualifiedName&, Document&);
+    static Ref<SVGFEFuncGElement> create(const QualifiedName&, Document&);
 
 private:
     SVGFEFuncGElement(const QualifiedName&, Document&);
 };
 
 } // namespace WebCore
-
-#endif
