@@ -40,13 +40,22 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
         const altToolTip = WI.UIString("Stop recording canvas actions");
         this._recordButtonNavigationItem = new WI.ToggleButtonNavigationItem("record-start-stop", toolTip, altToolTip, "Images/Record.svg", "Images/Stop.svg", 13, 13);
         this._recordButtonNavigationItem.enabled = false;
+        this._recordButtonNavigationItem.buttonStyle = WI.ButtonNavigationItem.Style.ImageAndText;
+        this._recordButtonNavigationItem.label = WI.UIString("Start");
         this._recordButtonNavigationItem.addEventListener(WI.ButtonNavigationItem.Event.Clicked, this._toggleRecording, this);
         this._navigationBar.addNavigationItem(this._recordButtonNavigationItem);
 
+        this._navigationBar.addNavigationItem(new WI.DividerNavigationItem);
+
+        let importButtonNavigationItem = new WI.ButtonNavigationItem("import-recording", WI.UIString("Import"), "Images/Import.svg", 15, 15);
+        importButtonNavigationItem.buttonStyle = WI.ButtonNavigationItem.Style.ImageAndText;
+        importButtonNavigationItem.visibilityPriority = WI.NavigationItem.VisibilityPriority.Low;
+        importButtonNavigationItem.addEventListener(WI.ButtonNavigationItem.Event.Clicked, this._handleImportButtonNavigationItemClicked, this);
+        this._navigationBar.addNavigationItem(importButtonNavigationItem);
+
         this.addSubview(this._navigationBar);
 
-        const suppressFiltering = true;
-        this._canvasTreeOutline = this.createContentTreeOutline(suppressFiltering);
+        this._canvasTreeOutline = this.createContentTreeOutline({suppressFiltering: true});
         this._canvasTreeOutline.element.classList.add("canvas");
 
         this._recordingNavigationBar = new WI.NavigationBar;
@@ -62,14 +71,12 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
         this._recordingTreeOutline.customIndent = true;
         this._recordingTreeOutline.registerScrollVirtualizer(this._recordingContentContainer, 20);
 
-        this._canvasTreeOutline.addEventListener(WI.TreeOutline.Event.SelectionDidChange, this._treeOutlineSelectionDidChange, this);
-        this._recordingTreeOutline.addEventListener(WI.TreeOutline.Event.SelectionDidChange, this._treeOutlineSelectionDidChange, this);
+        this._canvasTreeOutline.addEventListener(WI.TreeOutline.Event.SelectionDidChange, this._treeSelectionDidChange, this);
+        this._recordingTreeOutline.addEventListener(WI.TreeOutline.Event.SelectionDidChange, this._treeSelectionDidChange, this);
 
-        WI.canvasManager.addEventListener(WI.CanvasManager.Event.RecordingStarted, this._updateRecordNavigationItem, this);
-        WI.canvasManager.addEventListener(WI.CanvasManager.Event.RecordingStopped, this._updateRecordNavigationItem, this);
+        this._recordingProcessingOptionsContainer = null;
 
-        this._recordingProcessPromise = null;
-        this._recordingProcessSpinner = null;
+        this._selectedRecordingActionIndex = NaN;
     }
 
     // Public
@@ -84,11 +91,15 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
         if (this._canvas === canvas)
             return;
 
-        if (this._canvas)
+        if (this._canvas) {
+            this._canvas.removeEventListener(null, null, this);
             this._canvas.recordingCollection.removeEventListener(null, null, this);
+        }
 
         this._canvas = canvas;
         if (this._canvas) {
+            this._canvas.addEventListener(WI.Canvas.Event.RecordingStarted, this._updateRecordNavigationItem, this);
+            this._canvas.addEventListener(WI.Canvas.Event.RecordingStopped, this._updateRecordNavigationItem, this);
             this._canvas.recordingCollection.addEventListener(WI.Collection.Event.ItemAdded, this._recordingAdded, this);
             this._canvas.recordingCollection.addEventListener(WI.Collection.Event.ItemRemoved, this._recordingRemoved, this);
         }
@@ -103,16 +114,30 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
         if (recording === this._recording)
             return;
 
+        if (this._recording)
+            this._recording.removeEventListener(null, null, this);
+
         if (recording)
             this.canvas = recording.source;
 
         this._recording = recording;
+
+        if (this._recording && !this._recording.ready) {
+            this._recording.addEventListener(WI.Recording.Event.ProcessedAction, this._handleRecordingProcessedAction, this);
+            this._recording.addEventListener(WI.Recording.Event.StartProcessingFrame, this._handleRecordingStartProcessingFrame, this);
+        }
+
+        this._updateRecordNavigationItem();
+        this._updateRecordingScopeBar();
         this._recordingChanged();
     }
 
     set action(action)
     {
-        if (!this._recording || this._recordingProcessPromise)
+        if (!this._recording)
+            return;
+
+        if (action === this._recording.actions[this._selectedRecordingActionIndex])
             return;
 
         let selectedTreeElement = this._recordingTreeOutline.selectedTreeElement;
@@ -138,14 +163,65 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
         const omitFocus = false;
         const selectedByUser = false;
         treeElement.revealAndSelect(omitFocus, selectedByUser);
+
+        this._selectedRecordingActionIndex = this._recording.actions.indexOf(action);
+    }
+
+    updateRepresentedObjects()
+    {
+        let objects = this.contentBrowser.currentRepresentedObjects;
+
+        let canvas = objects.find((object) => object instanceof WI.Canvas);
+        if (canvas) {
+            this.canvas = canvas;
+            return;
+        }
+
+        let shaderProgram = objects.find((object) => object instanceof WI.ShaderProgram);
+        if (shaderProgram) {
+            this.canvas = shaderProgram.canvas;
+            let treeElement = this._canvasTreeOutline.findTreeElement(shaderProgram);
+            const omitFocus = false;
+            const selectedByUser = false;
+            treeElement.revealAndSelect(omitFocus, selectedByUser);
+            return;
+        }
+
+        let recording = objects.find((object) => object instanceof WI.Recording);
+        if (recording) {
+            this.canvas = recording.source;
+
+            this.recording = recording;
+
+            let recordingAction = objects.find((object) => object instanceof WI.RecordingAction);
+            if (recordingAction !== recording[WI.CanvasSidebarPanel.SelectedActionSymbol])
+                this.action = recordingAction;
+
+            return;
+        }
+
+        this.canvas = null;
+        this.recording = null;
     }
 
     shown()
     {
         super.shown();
 
-        this.contentBrowser.addEventListener(WI.ContentBrowser.Event.CurrentRepresentedObjectsDidChange, this._currentRepresentedObjectsDidChange, this);
-        this._currentRepresentedObjectsDidChange();
+        this.contentBrowser.addEventListener(WI.ContentBrowser.Event.CurrentRepresentedObjectsDidChange, this.updateRepresentedObjects, this);
+        this.updateRepresentedObjects();
+
+        if (this._recording) {
+            this._recordingTreeOutline.updateVirtualizedElementsDebouncer.force();
+
+            let action = this._recording[WI.CanvasSidebarPanel.SelectedActionSymbol];
+            let treeElement = this._recordingTreeOutline.findTreeElement(action);
+            if (treeElement) {
+                const omitFocus = false;
+                const selectedByUser = false;
+                treeElement.revealAndSelect(omitFocus, selectedByUser);
+            }
+        }
     }
 
     hidden()
@@ -165,6 +241,11 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
 
     // Protected
 
+    get scrollElement()
+    {
+        return this._recordingContentContainer;
+    }
+
     hasCustomFilters()
     {
         return true;
@@ -205,18 +286,15 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
     _recordingAdded(event)
     {
         this.recording = event.data.item;
-
-        this._updateRecordNavigationItem();
-        this._updateRecordingScopeBar();
     }
 
     _recordingRemoved(event)
     {
+        this._updateRecordingScopeBar();
+
         let recording = event.data.item;
         if (recording === this.recording)
             this.recording = this._canvas ? Array.from(this._canvas.recordingCollection).lastValue : null;
-
-        this._updateRecordingScopeBar();
     }
 
     _scopeBarSelectionChanged()
@@ -230,48 +308,22 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
         if (!this._canvas)
             return;
 
-        if (this._canvas.isRecording)
-            WI.canvasManager.stopRecording();
-        else if (!WI.canvasManager.recordingCanvas) {
+        if (this._canvas.recordingActive)
+            this._canvas.stopRecording();
+        else {
             let singleFrame = event.data.nativeEvent.shiftKey;
-            WI.canvasManager.startRecording(this._canvas, singleFrame);
+            this._canvas.startRecording(singleFrame);
         }
     }
 
-    _currentRepresentedObjectsDidChange(event)
+    _handleImportButtonNavigationItemClicked(event)
     {
-        let objects = this.contentBrowser.currentRepresentedObjects;
-
-        let canvas = objects.find((object) => object instanceof WI.Canvas);
-        if (canvas) {
-            this.canvas = canvas;
-            return;
-        }
-
-        let shaderProgram = objects.find((object) => object instanceof WI.ShaderProgram);
-        if (shaderProgram) {
-            this.canvas = shaderProgram.canvas;
-            let treeElement = this._canvasTreeOutline.findTreeElement(shaderProgram);
-            const omitFocus = false;
-            const selectedByUser = false;
-            treeElement.revealAndSelect(omitFocus, selectedByUser);
-            return;
-        }
-
-        let recording = objects.find((object) => object instanceof WI.Recording);
-        if (recording) {
-            recording[WI.CanvasSidebarPanel.SelectedActionSymbol] = objects.find((object) => object instanceof WI.RecordingAction);
-            this.recording = recording;
-            return;
-        }
-
-        this.canvas = null;
-        this.recording = null;
+        WI.FileUtilities.importJSON((result) => WI.canvasManager.processJSON(result));
     }
 
-    _treeOutlineSelectionDidChange(event)
+    _treeSelectionDidChange(event)
     {
-        let treeElement = event.data.selectedElement;
+        let treeElement = event.target.selectedTreeElement;
         if (!treeElement)
             return;
 
@@ -294,8 +346,13 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
 
         const onlyExisting = true;
         let recordingContentView = this.contentBrowser.contentViewForRepresentedObject(this._recording, onlyExisting);
-        if (recordingContentView)
-            recordingContentView.updateActionIndex(treeElement.index);
+        if (!recordingContentView)
+            return;
+
+        this.contentBrowser.showContentView(recordingContentView);
+
+        this._selectedRecordingActionIndex = treeElement.index;
+        recordingContentView.updateActionIndex(this._selectedRecordingActionIndex);
     }
 
     _canvasChanged()
@@ -316,7 +373,7 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
         const selectedByUser = false;
         canvasTreeElement.revealAndSelect(omitFocus, selectedByUser);
 
-        if (WI.Canvas.ContextType.Canvas2D || this._canvas.contextType === WI.Canvas.ContextType.WebGL)
+        if (WI.Canvas.ContextType.Canvas2D || this._canvas.contextType === WI.Canvas.ContextType.WebGL || this._canvas.contextType === WI.Canvas.ContextType.WebGL2)
             this._recordButtonNavigationItem.enabled = true;
 
         this.recording = null;
@@ -326,78 +383,102 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
     {
         this._recordingTreeOutline.removeChildren();
 
+        this._selectedRecordingActionIndex = NaN;
+
+        if (this._recordingProcessingOptionsContainer) {
+            this._recordingProcessingOptionsContainer.remove();
+            this._recordingProcessingOptionsContainer = null;
+        }
+
         if (!this._recording)
             return;
 
-        if (!this._recordingProcessSpinner) {
-            this._recordingProcessSpinner = new WI.IndeterminateProgressSpinner;
-            this._recordingContentContainer.appendChild(this._recordingProcessSpinner.element);
+        if (!this._recording.ready) {
+            if (!this._recording.processing)
+                this._recording.startProcessing();
+
+            if (!this._recordingProcessingOptionsContainer) {
+                this._recordingProcessingOptionsContainer = this._recordingContentContainer.appendChild(document.createElement("div"));
+                this._recordingProcessingOptionsContainer.classList.add("recording-processing-options");
+
+                let createPauseButton = () => {
+                    let spinner = new WI.IndeterminateProgressSpinner;
+                    this._recordingProcessingOptionsContainer.appendChild(spinner.element);
+
+                    let pauseButton = this._recordingProcessingOptionsContainer.appendChild(document.createElement("button"));
+                    pauseButton.textContent = WI.UIString("Pause Processing");
+                    pauseButton.addEventListener("click", (event) => {
+                        this._recording.stopProcessing();
+
+                        spinner.element.remove();
+                        pauseButton.remove();
+                        createResumeButton();
+                    });
+                };
+
+                let createResumeButton = () => {
+                    let resumeButton = this._recordingProcessingOptionsContainer.appendChild(document.createElement("button"));
+                    resumeButton.textContent = WI.UIString("Resume Processing");
+                    resumeButton.addEventListener("click", (event) => {
+                        this._recording.startProcessing();
+
+                        resumeButton.remove();
+                        createPauseButton();
+                    });
+                };
+
+                if (this._recording.processing)
+                    createPauseButton();
+                else
+                    createResumeButton();
+            }
         }
 
         this.contentBrowser.showContentViewForRepresentedObject(this._recording);
 
-        let recording = this._recording;
+        if (this._scopeBar) {
+            let scopeBarItem = this._scopeBar.item(this._recording.displayName);
+            console.assert(scopeBarItem, "Missing scopeBarItem for recording.", this._recording);
+            scopeBarItem.selected = true;
+        }
 
-        let promise = this._recording.process().then(() => {
-            if (recording !== this._recording || promise !== this._recordingProcessPromise)
+        let initialStateAction = this._recording.actions[0];
+        if (initialStateAction.ready && !this._recordingTreeOutline.getCachedTreeElement(initialStateAction)) {
+            this._recordingTreeOutline.appendChild(new WI.RecordingActionTreeElement(initialStateAction, 0, this._recording.type));
+
+            if (!this._recording[WI.CanvasSidebarPanel.SelectedActionSymbol])
+                this.action = initialStateAction;
+        }
+
+        let cumulativeActionIndex = 0;
+        this._recording.frames.forEach((frame, frameIndex) => {
+            if (!frame.actions[0].ready)
                 return;
 
-            if (this._recordingProcessSpinner) {
-                this._recordingProcessSpinner.element.remove();
-                this._recordingProcessSpinner = null;
+            let folder = this._recordingTreeOutline.getCachedTreeElement(frame);
+            if (!folder)
+                folder = this._createRecordingFrameTreeElement(frame, frameIndex, this._recordingTreeOutline);
+
+            for (let action of frame.actions) {
+                ++cumulativeActionIndex;
+
+                if (!action.ready || this._recordingTreeOutline.getCachedTreeElement(action))
+                    break;
+
+                this._createRecordingActionTreeElement(action, cumulativeActionIndex, folder);
             }
-
-            this._recordingTreeOutline.element.dataset.indent = Number.countDigits(this._recording.actions.length);
-
-            if (this._recording.actions[0] instanceof WI.RecordingInitialStateAction)
-                this._recordingTreeOutline.appendChild(new WI.RecordingActionTreeElement(this._recording.actions[0], 0, this._recording.type));
-
-            let cumulativeActionIndex = 1;
-            this._recording.frames.forEach((frame, frameIndex) => {
-                let folder = new WI.FolderTreeElement(WI.UIString("Frame %d").format((frameIndex + 1).toLocaleString()));
-                this._recordingTreeOutline.appendChild(folder);
-
-                for (let i = 0; i < frame.actions.length; ++i)
-                    folder.appendChild(new WI.RecordingActionTreeElement(frame.actions[i], cumulativeActionIndex + i, this._recording.type));
-
-                if (!isNaN(frame.duration)) {
-                    const higherResolution = true;
-                    folder.status = Number.secondsToString(frame.duration / 1000, higherResolution);
-                }
-
-                if (frame.incomplete)
-                    folder.subtitle = WI.UIString("Incomplete");
-
-                if (this._recording.frames.length === 1)
-                    folder.expand();
-
-                cumulativeActionIndex += frame.actions.length;
-            });
-
-            if (this._scopeBar) {
-                let scopeBarItem = this._scopeBar.item(this._recording.displayName);
-                console.assert(scopeBarItem, "Missing scopeBarItem for recording.", this._recording);
-                scopeBarItem.selected = true;
-            }
-
-            this.action = this._recording[WI.CanvasSidebarPanel.SelectedActionSymbol] || this._recording.actions[0];
-
-            this._recordingProcessPromise = null;
         });
-
-        this._recordingProcessPromise = promise;
     }
 
     _updateRecordNavigationItem()
     {
-        if (!this._canvas || !(this._canvas.contextType === WI.Canvas.ContextType.Canvas2D || this._canvas.contextType === WI.Canvas.ContextType.WebGL)) {
+        if (!this._canvas || !(this._canvas.contextType === WI.Canvas.ContextType.Canvas2D || this._canvas.contextType === WI.Canvas.ContextType.WebGL || this._canvas.contextType === WI.Canvas.ContextType.WebGL2)) {
             this._recordButtonNavigationItem.enabled = false;
             return;
         }
 
-        let isRecording = this._canvas.isRecording;
-        this._recordButtonNavigationItem.enabled = isRecording || !WI.canvasManager.recordingCanvas;
-        this._recordButtonNavigationItem.toggled = isRecording;
+        this._recordButtonNavigationItem.toggled = this._canvas.recordingActive;
+        this._recordButtonNavigationItem.label = this._recordButtonNavigationItem.toggled ? WI.UIString("Stop") : WI.UIString("Start");
     }
 
     _updateRecordingScopeBar()
@@ -411,14 +492,15 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
 
         this._recordingNavigationBar.element.classList.toggle("hidden", !this._canvas);
 
-        let hasRecordings = this._canvas && this._canvas.recordingCollection.size;
+        let hasRecordings = this._recording || (this._canvas && this._canvas.recordingCollection.size);
         this.element.classList.toggle("has-recordings", hasRecordings);
         if (!hasRecordings)
             return;
 
         let scopeBarItems = [];
         let selectedScopeBarItem = null;
-        for (let recording of this._canvas.recordingCollection) {
+
+        let createScopeBarItem = (recording) => {
             let scopeBarItem = new WI.ScopeBarItem(recording.displayName, recording.displayName);
             if (recording === this._recording)
                 selectedScopeBarItem = scopeBarItem;
@@ -426,24 +508,108 @@ WI.CanvasSidebarPanel = class CanvasSidebarPanel extends WI.NavigationSidebarPan
                 scopeBarItem.selected = false;
             scopeBarItem.__recording = recording;
             scopeBarItems.push(scopeBarItem);
+        };
+
+        if (this._canvas && this._canvas.recordingCollection) {
+            for (let recording of this._canvas.recordingCollection)
+                createScopeBarItem(recording);
         }
 
+        if (this._recording && (!this._canvas || !this._canvas.recordingCollection.has(this._recording)))
+            createScopeBarItem(this._recording);
+
         if (!selectedScopeBarItem) {
             selectedScopeBarItem = scopeBarItems[0];
 
-            const exclusive = true;
-            const className = null;
-            const hidden = true;
-            this._placeholderScopeBarItem = new WI.ScopeBarItem("canvas-recording-scope-bar-item-placeholder", WI.UIString("Recordings"), exclusive, className, hidden);
+            this._placeholderScopeBarItem = new WI.ScopeBarItem("canvas-recording-scope-bar-item-placeholder", WI.UIString("Recordings"), {exclusive: true, hidden: true});
             this._placeholderScopeBarItem.selected = true;
 
             scopeBarItems.unshift(this._placeholderScopeBarItem);
         }
 
-        this._scopeBar = new WI.ScopeBar("canvas-recordinga-scope-bar", scopeBarItems, selectedScopeBarItem, true);
+        this._scopeBar = new WI.ScopeBar("canvas-recording-scope-bar", scopeBarItems, selectedScopeBarItem, true);
         this._scopeBar.addEventListener(WI.ScopeBar.Event.SelectionChanged, this._scopeBarSelectionChanged, this);
         this._recordingNavigationBar.insertNavigationItem(this._scopeBar, 0);
     }
+
+    _createRecordingFrameTreeElement(frame, index, parent)
+    {
+        let folder = new WI.FolderTreeElement(WI.UIString("Frame %d").format((index + 1).toLocaleString()), frame);
+
+        if (!isNaN(frame.duration)) {
+            const higherResolution = true;
+            folder.status = Number.secondsToString(frame.duration / 1000, higherResolution);
+        }
+
+        parent.appendChild(folder);
+
+        return folder;
+    }
+
+    _createRecordingActionTreeElement(action, index, parent)
+    {
+        let treeElement = new WI.RecordingActionTreeElement(action, index, this._recording.type);
+
+        parent.appendChild(treeElement);
+
+        if (parent instanceof WI.FolderTreeElement && parent.representedObject instanceof WI.RecordingFrame) {
+            if (action !== parent.representedObject.actions.lastValue) {
+                parent.addClassName("processing");
+
+                if (!(parent.subtitle instanceof HTMLProgressElement))
+                    parent.subtitle = document.createElement("progress");
+
+                if (parent.statusElement)
+                    parent.subtitle.style.setProperty("width", `calc(100% - ${parent.statusElement.offsetWidth + 4}px`);
+
+                parent.subtitle.value = parent.representedObject.actions.indexOf(action) / parent.representedObject.actions.length;
+            } else {
+                parent.removeClassName("processing");
+                if (parent.representedObject.incomplete)
+                    parent.subtitle = WI.UIString("Incomplete");
+                else
+                    parent.subtitle = "";
+            }
+        }
+
+        if (action === this._recording[WI.CanvasSidebarPanel.SelectedActionSymbol])
+            this.action = action;
+
+        return treeElement;
+    }
+
+    _handleRecordingProcessedAction(event)
+    {
+        let {action, index} = event.data;
+
+        this._recordingTreeOutline.element.dataset.indent = Number.countDigits(index);
+
+        let isInitialStateAction = !index;
+
+        console.assert(isInitialStateAction || this._recordingTreeOutline.children.lastValue instanceof WI.FolderTreeElement, "There should be a WI.FolderTreeElement for the frame for this action.");
+        this._createRecordingActionTreeElement(action, index, isInitialStateAction ? this._recordingTreeOutline : this._recordingTreeOutline.children.lastValue);
+
+        if (!this._recording[WI.CanvasSidebarPanel.SelectedActionSymbol]) {
+            console.assert(action === this._recording.actions[0]);
+            this.action = this._recording.actions[0];
+        }
+
+        if (this._recording.ready) {
+            this._recording.removeEventListener(null, null, this);
+
+            if (this._recordingProcessingOptionsContainer) {
+                this._recordingProcessingOptionsContainer.remove();
+                this._recordingProcessingOptionsContainer = null;
+            }
+        }
+    }
+
+    _handleRecordingStartProcessingFrame(event)
+    {
+        let {frame, index} = event.data;
+
+        this._createRecordingFrameTreeElement(frame, index, this._recordingTreeOutline);
+    }
 };
 
 WI.CanvasSidebarPanel.SelectedActionSymbol = Symbol("selected-action");
