@@ -28,7 +28,6 @@
 #define WKAPICast_h
 
 #include "CacheModel.h"
-#include "FontSmoothingLevel.h"
 #include "HTTPCookieAcceptPolicy.h"
 #include "InjectedBundleHitTestResultMediaType.h"
 #include "PluginModuleInfo.h"
@@ -54,10 +53,12 @@
 namespace API {
 class ContentRuleList;
 class ContentRuleListStore;
+class InternalDebugFeature;
 class ExperimentalFeature;
 class FrameHandle;
 class FrameInfo;
 class HitTestResult;
+class MessageListener;
 class Navigation;
 class NavigationAction;
 class NavigationData;
@@ -137,6 +138,7 @@ WK_ADD_API_MAPPING(WKIconDatabaseRef, WebIconDatabase)
 WK_ADD_API_MAPPING(WKInspectorRef, WebInspectorProxy)
 WK_ADD_API_MAPPING(WKMediaSessionFocusManagerRef, WebMediaSessionFocusManager)
 WK_ADD_API_MAPPING(WKMediaSessionMetadataRef, WebMediaSessionMetadata)
+WK_ADD_API_MAPPING(WKMessageListenerRef, API::MessageListener)
 WK_ADD_API_MAPPING(WKNavigationActionRef, API::NavigationAction)
 WK_ADD_API_MAPPING(WKNavigationDataRef, API::NavigationData)
 WK_ADD_API_MAPPING(WKNavigationRef, API::Navigation)
@@ -204,25 +206,25 @@ inline CacheModel toCacheModel(WKCacheModel wkCacheModel)
 {
     switch (wkCacheModel) {
     case kWKCacheModelDocumentViewer:
-        return CacheModelDocumentViewer;
+        return CacheModel::DocumentViewer;
     case kWKCacheModelDocumentBrowser:
-        return CacheModelDocumentBrowser;
+        return CacheModel::DocumentBrowser;
     case kWKCacheModelPrimaryWebBrowser:
-        return CacheModelPrimaryWebBrowser;
+        return CacheModel::PrimaryWebBrowser;
     }
 
     ASSERT_NOT_REACHED();
-    return CacheModelDocumentViewer;
+    return CacheModel::DocumentViewer;
 }
 
 inline WKCacheModel toAPI(CacheModel cacheModel)
 {
     switch (cacheModel) {
-    case CacheModelDocumentViewer:
+    case CacheModel::DocumentViewer:
         return kWKCacheModelDocumentViewer;
-    case CacheModelDocumentBrowser:
+    case CacheModel::DocumentBrowser:
         return kWKCacheModelDocumentBrowser;
-    case CacheModelPrimaryWebBrowser:
+    case CacheModel::PrimaryWebBrowser:
         return kWKCacheModelPrimaryWebBrowser;
     }
     
@@ -249,41 +251,6 @@ inline WKProcessTerminationReason toAPI(ProcessTerminationReason reason)
     return kWKProcessTerminationReasonCrash;
 }
 
-inline FontSmoothingLevel toFontSmoothingLevel(WKFontSmoothingLevel wkLevel)
-{
-    switch (wkLevel) {
-    case kWKFontSmoothingLevelNoSubpixelAntiAliasing:
-        return FontSmoothingLevelNoSubpixelAntiAliasing;
-    case kWKFontSmoothingLevelLight:
-        return FontSmoothingLevelLight;
-    case kWKFontSmoothingLevelMedium:
-        return FontSmoothingLevelMedium;
-    case kWKFontSmoothingLevelStrong:
-        return FontSmoothingLevelStrong;
-    }
-
-    ASSERT_NOT_REACHED();
-    return FontSmoothingLevelMedium;
-}
-
-
-inline WKFontSmoothingLevel toAPI(FontSmoothingLevel level)
-{
-    switch (level) {
-    case FontSmoothingLevelNoSubpixelAntiAliasing:
-        return kWKFontSmoothingLevelNoSubpixelAntiAliasing;
-    case FontSmoothingLevelLight:
-        return kWKFontSmoothingLevelLight;
-    case FontSmoothingLevelMedium:
-        return kWKFontSmoothingLevelMedium;
-    case FontSmoothingLevelStrong:
-        return kWKFontSmoothingLevelStrong;
-    }
-
-    ASSERT_NOT_REACHED();
-    return kWKFontSmoothingLevelMedium;
-}
-
 inline WKEditableLinkBehavior toAPI(WebCore::EditableLinkBehavior behavior)
 {
     switch (behavior) {
@@ -402,29 +369,29 @@ inline HTTPCookieAcceptPolicy toHTTPCookieAcceptPolicy(WKHTTPCookieAcceptPolicy
 {
     switch (policy) {
     case kWKHTTPCookieAcceptPolicyAlways:
-        return HTTPCookieAcceptPolicyAlways;
+        return HTTPCookieAcceptPolicy::AlwaysAccept;
     case kWKHTTPCookieAcceptPolicyNever:
-        return HTTPCookieAcceptPolicyNever;
+        return HTTPCookieAcceptPolicy::Never;
     case kWKHTTPCookieAcceptPolicyOnlyFromMainDocumentDomain:
-        return HTTPCookieAcceptPolicyOnlyFromMainDocumentDomain;
+        return HTTPCookieAcceptPolicy::OnlyFromMainDocumentDomain;
     case kWKHTTPCookieAcceptPolicyExclusivelyFromMainDocumentDomain:
-        return HTTPCookieAcceptPolicyExclusivelyFromMainDocumentDomain;
+        return HTTPCookieAcceptPolicy::ExclusivelyFromMainDocumentDomain;
     }
 
     ASSERT_NOT_REACHED();
-    return HTTPCookieAcceptPolicyAlways;
+    return HTTPCookieAcceptPolicy::AlwaysAccept;
 }
 
 inline WKHTTPCookieAcceptPolicy toAPI(HTTPCookieAcceptPolicy policy)
 {
     switch (policy) {
-    case HTTPCookieAcceptPolicyAlways:
+    case HTTPCookieAcceptPolicy::AlwaysAccept:
         return kWKHTTPCookieAcceptPolicyAlways;
-    case HTTPCookieAcceptPolicyNever:
+    case HTTPCookieAcceptPolicy::Never:
         return kWKHTTPCookieAcceptPolicyNever;
-    case HTTPCookieAcceptPolicyOnlyFromMainDocumentDomain:
+    case HTTPCookieAcceptPolicy::OnlyFromMainDocumentDomain:
         return kWKHTTPCookieAcceptPolicyOnlyFromMainDocumentDomain;
-    case HTTPCookieAcceptPolicyExclusivelyFromMainDocumentDomain:
+    case HTTPCookieAcceptPolicy::ExclusivelyFromMainDocumentDomain:
         return kWKHTTPCookieAcceptPolicyExclusivelyFromMainDocumentDomain;
     }
 
